# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: yamcs/protobuf/plists/plists_service.proto

from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from google.protobuf import empty_pb2 as google_dot_protobuf_dot_empty__pb2
from yamcs.api import annotations_pb2 as yamcs_dot_api_dot_annotations__pb2
from yamcs.protobuf.plists import plists_pb2 as yamcs_dot_protobuf_dot_plists_dot_plists__pb2


DESCRIPTOR = _descriptor.FileDescriptor(
  name='yamcs/protobuf/plists/plists_service.proto',
  package='yamcs.protobuf.plists',
  syntax='proto2',
  serialized_options=b'\n\031org.yamcs.protobuf.plistsB\032ParameterListsServiceProtoP\001',
  serialized_pb=b'\n*yamcs/protobuf/plists/plists_service.proto\x12\x15yamcs.protobuf.plists\x1a\x1bgoogle/protobuf/empty.proto\x1a\x1byamcs/api/annotations.proto\x1a\"yamcs/protobuf/plists/plists.proto\"-\n\x19ListParameterListsRequest\x12\x10\n\x08instance\x18\x01 \x01(\t\"9\n\x17GetParameterListRequest\x12\x10\n\x08instance\x18\x01 \x01(\t\x12\x0c\n\x04list\x18\x02 \x01(\t\"U\n\x1aListParameterListsResponse\x12\x37\n\x05lists\x18\x01 \x03(\x0b\x32(.yamcs.protobuf.plists.ParameterListInfo\"c\n\x1a\x43reateParameterListRequest\x12\x10\n\x08instance\x18\x01 \x01(\t\x12\x0c\n\x04name\x18\x02 \x01(\t\x12\x13\n\x0b\x64\x65scription\x18\x03 \x01(\t\x12\x10\n\x08patterns\x18\x04 \x03(\t\"\xa4\x01\n\x1aUpdateParameterListRequest\x12\x10\n\x08instance\x18\x01 \x01(\t\x12\x0c\n\x04list\x18\x02 \x01(\t\x12\x0c\n\x04name\x18\x03 \x01(\t\x12\x13\n\x0b\x64\x65scription\x18\x04 \x01(\t\x12\x43\n\x11patternDefinition\x18\x05 \x01(\x0b\x32(.yamcs.protobuf.plists.PatternDefinition\"%\n\x11PatternDefinition\x12\x10\n\x08patterns\x18\x01 \x03(\t\"<\n\x1a\x44\x65leteParameterListRequest\x12\x10\n\x08instance\x18\x01 \x01(\t\x12\x0c\n\x04list\x18\x02 \x01(\t2\xcd\x06\n\x11ParameterListsApi\x12\xa6\x01\n\x12ListParameterLists\x12\x30.yamcs.protobuf.plists.ListParameterListsRequest\x1a\x31.yamcs.protobuf.plists.ListParameterListsResponse\"+\x8a\x92\x03\'\n%/api/parameter-lists/{instance}/lists\x12\xa0\x01\n\x10GetParameterList\x12..yamcs.protobuf.plists.GetParameterListRequest\x1a(.yamcs.protobuf.plists.ParameterListInfo\"2\x8a\x92\x03.\n,/api/parameter-lists/{instance}/lists/{list}\x12\xa2\x01\n\x13\x43reateParameterList\x12\x31.yamcs.protobuf.plists.CreateParameterListRequest\x1a(.yamcs.protobuf.plists.ParameterListInfo\".\x8a\x92\x03*\x1a%/api/parameter-lists/{instance}/lists:\x01*\x12\xa9\x01\n\x13UpdateParameterList\x12\x31.yamcs.protobuf.plists.UpdateParameterListRequest\x1a(.yamcs.protobuf.plists.ParameterListInfo\"5\x8a\x92\x03\x31*,/api/parameter-lists/{instance}/lists/{list}:\x01*\x12\x94\x01\n\x13\x44\x65leteParameterList\x12\x31.yamcs.protobuf.plists.DeleteParameterListRequest\x1a\x16.google.protobuf.Empty\"2\x8a\x92\x03.\",/api/parameter-lists/{instance}/lists/{list}\x1a\x04\xd0\x80\x01\x01\x42\x39\n\x19org.yamcs.protobuf.plistsB\x1aParameterListsServiceProtoP\x01'
  ,
  dependencies=[google_dot_protobuf_dot_empty__pb2.DESCRIPTOR,yamcs_dot_api_dot_annotations__pb2.DESCRIPTOR,yamcs_dot_protobuf_dot_plists_dot_plists__pb2.DESCRIPTOR,])




_LISTPARAMETERLISTSREQUEST = _descriptor.Descriptor(
  name='ListParameterListsRequest',
  full_name='yamcs.protobuf.plists.ListParameterListsRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='instance', full_name='yamcs.protobuf.plists.ListParameterListsRequest.instance', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=163,
  serialized_end=208,
)


_GETPARAMETERLISTREQUEST = _descriptor.Descriptor(
  name='GetParameterListRequest',
  full_name='yamcs.protobuf.plists.GetParameterListRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='instance', full_name='yamcs.protobuf.plists.GetParameterListRequest.instance', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='list', full_name='yamcs.protobuf.plists.GetParameterListRequest.list', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=210,
  serialized_end=267,
)


_LISTPARAMETERLISTSRESPONSE = _descriptor.Descriptor(
  name='ListParameterListsResponse',
  full_name='yamcs.protobuf.plists.ListParameterListsResponse',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='lists', full_name='yamcs.protobuf.plists.ListParameterListsResponse.lists', index=0,
      number=1, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=269,
  serialized_end=354,
)


_CREATEPARAMETERLISTREQUEST = _descriptor.Descriptor(
  name='CreateParameterListRequest',
  full_name='yamcs.protobuf.plists.CreateParameterListRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='instance', full_name='yamcs.protobuf.plists.CreateParameterListRequest.instance', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='name', full_name='yamcs.protobuf.plists.CreateParameterListRequest.name', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='description', full_name='yamcs.protobuf.plists.CreateParameterListRequest.description', index=2,
      number=3, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='patterns', full_name='yamcs.protobuf.plists.CreateParameterListRequest.patterns', index=3,
      number=4, type=9, cpp_type=9, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=356,
  serialized_end=455,
)


_UPDATEPARAMETERLISTREQUEST = _descriptor.Descriptor(
  name='UpdateParameterListRequest',
  full_name='yamcs.protobuf.plists.UpdateParameterListRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='instance', full_name='yamcs.protobuf.plists.UpdateParameterListRequest.instance', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='list', full_name='yamcs.protobuf.plists.UpdateParameterListRequest.list', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='name', full_name='yamcs.protobuf.plists.UpdateParameterListRequest.name', index=2,
      number=3, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='description', full_name='yamcs.protobuf.plists.UpdateParameterListRequest.description', index=3,
      number=4, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='patternDefinition', full_name='yamcs.protobuf.plists.UpdateParameterListRequest.patternDefinition', index=4,
      number=5, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=458,
  serialized_end=622,
)


_PATTERNDEFINITION = _descriptor.Descriptor(
  name='PatternDefinition',
  full_name='yamcs.protobuf.plists.PatternDefinition',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='patterns', full_name='yamcs.protobuf.plists.PatternDefinition.patterns', index=0,
      number=1, type=9, cpp_type=9, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=624,
  serialized_end=661,
)


_DELETEPARAMETERLISTREQUEST = _descriptor.Descriptor(
  name='DeleteParameterListRequest',
  full_name='yamcs.protobuf.plists.DeleteParameterListRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='instance', full_name='yamcs.protobuf.plists.DeleteParameterListRequest.instance', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='list', full_name='yamcs.protobuf.plists.DeleteParameterListRequest.list', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=663,
  serialized_end=723,
)

_LISTPARAMETERLISTSRESPONSE.fields_by_name['lists'].message_type = yamcs_dot_protobuf_dot_plists_dot_plists__pb2._PARAMETERLISTINFO
_UPDATEPARAMETERLISTREQUEST.fields_by_name['patternDefinition'].message_type = _PATTERNDEFINITION
DESCRIPTOR.message_types_by_name['ListParameterListsRequest'] = _LISTPARAMETERLISTSREQUEST
DESCRIPTOR.message_types_by_name['GetParameterListRequest'] = _GETPARAMETERLISTREQUEST
DESCRIPTOR.message_types_by_name['ListParameterListsResponse'] = _LISTPARAMETERLISTSRESPONSE
DESCRIPTOR.message_types_by_name['CreateParameterListRequest'] = _CREATEPARAMETERLISTREQUEST
DESCRIPTOR.message_types_by_name['UpdateParameterListRequest'] = _UPDATEPARAMETERLISTREQUEST
DESCRIPTOR.message_types_by_name['PatternDefinition'] = _PATTERNDEFINITION
DESCRIPTOR.message_types_by_name['DeleteParameterListRequest'] = _DELETEPARAMETERLISTREQUEST
_sym_db.RegisterFileDescriptor(DESCRIPTOR)

ListParameterListsRequest = _reflection.GeneratedProtocolMessageType('ListParameterListsRequest', (_message.Message,), {
  'DESCRIPTOR' : _LISTPARAMETERLISTSREQUEST,
  '__module__' : 'yamcs.protobuf.plists.plists_service_pb2'
  # @@protoc_insertion_point(class_scope:yamcs.protobuf.plists.ListParameterListsRequest)
  })
_sym_db.RegisterMessage(ListParameterListsRequest)

GetParameterListRequest = _reflection.GeneratedProtocolMessageType('GetParameterListRequest', (_message.Message,), {
  'DESCRIPTOR' : _GETPARAMETERLISTREQUEST,
  '__module__' : 'yamcs.protobuf.plists.plists_service_pb2'
  # @@protoc_insertion_point(class_scope:yamcs.protobuf.plists.GetParameterListRequest)
  })
_sym_db.RegisterMessage(GetParameterListRequest)

ListParameterListsResponse = _reflection.GeneratedProtocolMessageType('ListParameterListsResponse', (_message.Message,), {
  'DESCRIPTOR' : _LISTPARAMETERLISTSRESPONSE,
  '__module__' : 'yamcs.protobuf.plists.plists_service_pb2'
  # @@protoc_insertion_point(class_scope:yamcs.protobuf.plists.ListParameterListsResponse)
  })
_sym_db.RegisterMessage(ListParameterListsResponse)

CreateParameterListRequest = _reflection.GeneratedProtocolMessageType('CreateParameterListRequest', (_message.Message,), {
  'DESCRIPTOR' : _CREATEPARAMETERLISTREQUEST,
  '__module__' : 'yamcs.protobuf.plists.plists_service_pb2'
  # @@protoc_insertion_point(class_scope:yamcs.protobuf.plists.CreateParameterListRequest)
  })
_sym_db.RegisterMessage(CreateParameterListRequest)

UpdateParameterListRequest = _reflection.GeneratedProtocolMessageType('UpdateParameterListRequest', (_message.Message,), {
  'DESCRIPTOR' : _UPDATEPARAMETERLISTREQUEST,
  '__module__' : 'yamcs.protobuf.plists.plists_service_pb2'
  # @@protoc_insertion_point(class_scope:yamcs.protobuf.plists.UpdateParameterListRequest)
  })
_sym_db.RegisterMessage(UpdateParameterListRequest)

PatternDefinition = _reflection.GeneratedProtocolMessageType('PatternDefinition', (_message.Message,), {
  'DESCRIPTOR' : _PATTERNDEFINITION,
  '__module__' : 'yamcs.protobuf.plists.plists_service_pb2'
  # @@protoc_insertion_point(class_scope:yamcs.protobuf.plists.PatternDefinition)
  })
_sym_db.RegisterMessage(PatternDefinition)

DeleteParameterListRequest = _reflection.GeneratedProtocolMessageType('DeleteParameterListRequest', (_message.Message,), {
  'DESCRIPTOR' : _DELETEPARAMETERLISTREQUEST,
  '__module__' : 'yamcs.protobuf.plists.plists_service_pb2'
  # @@protoc_insertion_point(class_scope:yamcs.protobuf.plists.DeleteParameterListRequest)
  })
_sym_db.RegisterMessage(DeleteParameterListRequest)


DESCRIPTOR._options = None

_PARAMETERLISTSAPI = _descriptor.ServiceDescriptor(
  name='ParameterListsApi',
  full_name='yamcs.protobuf.plists.ParameterListsApi',
  file=DESCRIPTOR,
  index=0,
  serialized_options=b'\320\200\001\001',
  serialized_start=726,
  serialized_end=1571,
  methods=[
  _descriptor.MethodDescriptor(
    name='ListParameterLists',
    full_name='yamcs.protobuf.plists.ParameterListsApi.ListParameterLists',
    index=0,
    containing_service=None,
    input_type=_LISTPARAMETERLISTSREQUEST,
    output_type=_LISTPARAMETERLISTSRESPONSE,
    serialized_options=b'\212\222\003\'\n%/api/parameter-lists/{instance}/lists',
  ),
  _descriptor.MethodDescriptor(
    name='GetParameterList',
    full_name='yamcs.protobuf.plists.ParameterListsApi.GetParameterList',
    index=1,
    containing_service=None,
    input_type=_GETPARAMETERLISTREQUEST,
    output_type=yamcs_dot_protobuf_dot_plists_dot_plists__pb2._PARAMETERLISTINFO,
    serialized_options=b'\212\222\003.\n,/api/parameter-lists/{instance}/lists/{list}',
  ),
  _descriptor.MethodDescriptor(
    name='CreateParameterList',
    full_name='yamcs.protobuf.plists.ParameterListsApi.CreateParameterList',
    index=2,
    containing_service=None,
    input_type=_CREATEPARAMETERLISTREQUEST,
    output_type=yamcs_dot_protobuf_dot_plists_dot_plists__pb2._PARAMETERLISTINFO,
    serialized_options=b'\212\222\003*\032%/api/parameter-lists/{instance}/lists:\001*',
  ),
  _descriptor.MethodDescriptor(
    name='UpdateParameterList',
    full_name='yamcs.protobuf.plists.ParameterListsApi.UpdateParameterList',
    index=3,
    containing_service=None,
    input_type=_UPDATEPARAMETERLISTREQUEST,
    output_type=yamcs_dot_protobuf_dot_plists_dot_plists__pb2._PARAMETERLISTINFO,
    serialized_options=b'\212\222\0031*,/api/parameter-lists/{instance}/lists/{list}:\001*',
  ),
  _descriptor.MethodDescriptor(
    name='DeleteParameterList',
    full_name='yamcs.protobuf.plists.ParameterListsApi.DeleteParameterList',
    index=4,
    containing_service=None,
    input_type=_DELETEPARAMETERLISTREQUEST,
    output_type=google_dot_protobuf_dot_empty__pb2._EMPTY,
    serialized_options=b'\212\222\003.\",/api/parameter-lists/{instance}/lists/{list}',
  ),
])
_sym_db.RegisterServiceDescriptor(_PARAMETERLISTSAPI)

DESCRIPTOR.services_by_name['ParameterListsApi'] = _PARAMETERLISTSAPI

# @@protoc_insertion_point(module_scope)



import warnings

warnings.warn(
    "The module 'yamcs.storage.model' is deprecated. "
    "Import classes from 'yamcs.client' instead.",
    DeprecationWarning,
    stacklevel=2,
)

from yamcs.client.storage.model import Bucket, ObjectInfo, ObjectListing  # noqa



from yamcs.client import clientversion  # noqa
from yamcs.client.activities import *  # noqa
from yamcs.client.archive.client import *  # noqa
from yamcs.client.archive.model import *  # noqa
from yamcs.client.core import *  # noqa
from yamcs.client.core.auth import *  # noqa
from yamcs.client.core.context import *  # noqa
from yamcs.client.core.exceptions import *  # noqa
from yamcs.client.core.futures import *  # noqa
from yamcs.client.core.helpers import *  # noqa
from yamcs.client.core.pagination import *  # noqa
from yamcs.client.core.subscriptions import *  # noqa
from yamcs.client.filetransfer.client import *  # noqa
from yamcs.client.filetransfer.model import *  # noqa
from yamcs.client.links.client import *  # noqa
from yamcs.client.links.model import *  # noqa
from yamcs.client.mdb.client import *  # noqa
from yamcs.client.mdb.model import *  # noqa
from yamcs.client.model import *  # noqa
from yamcs.client.storage.client import *  # noqa
from yamcs.client.storage.model import *  # noqa
from yamcs.client.tco.client import *  # noqa
from yamcs.client.tco.model import *  # noqa
from yamcs.client.timeline.client import *  # noqa
from yamcs.client.timeline.model import *  # noqa
from yamcs.client.tmtc.client import *  # noqa
from yamcs.client.tmtc.model import *  # noqa



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: yamcs/protobuf/archive/parameter_archive_service.proto

from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from google.protobuf import empty_pb2 as google_dot_protobuf_dot_empty__pb2
from google.protobuf import timestamp_pb2 as google_dot_protobuf_dot_timestamp__pb2
from yamcs.api import annotations_pb2 as yamcs_dot_api_dot_annotations__pb2
from yamcs.protobuf import yamcs_pb2 as yamcs_dot_protobuf_dot_yamcs__pb2


DESCRIPTOR = _descriptor.FileDescriptor(
  name='yamcs/protobuf/archive/parameter_archive_service.proto',
  package='yamcs.protobuf.archive',
  syntax='proto2',
  serialized_options=b'\n\022org.yamcs.protobufB\034ParameterArchiveServiceProtoP\001',
  serialized_pb=b'\n6yamcs/protobuf/archive/parameter_archive_service.proto\x12\x16yamcs.protobuf.archive\x1a\x1bgoogle/protobuf/empty.proto\x1a\x1fgoogle/protobuf/timestamp.proto\x1a\x1byamcs/api/annotations.proto\x1a\x1ayamcs/protobuf/yamcs.proto\"/\n\x1bSubscribeBackfillingRequest\x12\x10\n\x08instance\x18\x01 \x01(\t\"\xfe\x01\n\x18SubscribeBackfillingData\x12W\n\x08\x66inished\x18\x01 \x03(\x0b\x32\x45.yamcs.protobuf.archive.SubscribeBackfillingData.BackfillFinishedInfo\x1a\x88\x01\n\x14\x42\x61\x63kfillFinishedInfo\x12)\n\x05start\x18\x01 \x01(\x0b\x32\x1a.google.protobuf.Timestamp\x12(\n\x04stop\x18\x02 \x01(\x0b\x32\x1a.google.protobuf.Timestamp\x12\x1b\n\x13processedParameters\x18\x03 \x01(\x04\"|\n\x13RebuildRangeRequest\x12\x10\n\x08instance\x18\x01 \x01(\t\x12)\n\x05start\x18\x02 \x01(\x0b\x32\x1a.google.protobuf.Timestamp\x12(\n\x04stop\x18\x03 \x01(\x0b\x32\x1a.google.protobuf.Timestamp\" \n\x0cPurgeRequest\x12\x10\n\x08instance\x18\x01 \x01(\t\"a\n GetArchivedParametersInfoRequest\x12\x10\n\x08instance\x18\x01 \x01(\t\x12\x0e\n\x06\x66ilter\x18\x04 \x01(\t\x12\r\n\x05limit\x18\x07 \x01(\x05\x12\x0c\n\x04next\x18\x08 \x01(\t\"\x99\x01\n#GetArchivedParameterSegmentsRequest\x12\x10\n\x08instance\x18\x01 \x01(\t\x12\x0b\n\x03pid\x18\x02 \x01(\r\x12)\n\x05start\x18\x03 \x01(\x0b\x32\x1a.google.protobuf.Timestamp\x12(\n\x04stop\x18\x04 \x01(\x0b\x32\x1a.google.protobuf.Timestamp\"\x9f\x01\n\x15\x41rchivedParameterInfo\x12\x0b\n\x03pid\x18\x01 \x01(\r\x12\x11\n\tparameter\x18\x02 \x01(\t\x12+\n\x07rawType\x18\x03 \x01(\x0e\x32\x1a.yamcs.protobuf.Value.Type\x12+\n\x07\x65ngType\x18\x04 \x01(\x0e\x32\x1a.yamcs.protobuf.Value.Type\x12\x0c\n\x04gids\x18\x05 \x03(\r\"x\n\x1e\x41rchivedParametersInfoResponse\x12;\n\x04pids\x18\x01 \x03(\x0b\x32-.yamcs.protobuf.archive.ArchivedParameterInfo\x12\x19\n\x11\x63ontinuationToken\x18\x02 \x01(\t\"\x91\x01\n\x1b\x41rchiveParameterSegmentInfo\x12\x0f\n\x07groupId\x18\x01 \x01(\r\x12)\n\x05start\x18\x02 \x01(\x0b\x32\x1a.google.protobuf.Timestamp\x12\'\n\x03\x65nd\x18\x03 \x01(\x0b\x32\x1a.google.protobuf.Timestamp\x12\r\n\x05\x63ount\x18\x04 \x01(\r\"\xb0\x01\n!ArchivedParameterSegmentsResponse\x12\x44\n\rparameterInfo\x18\x01 \x01(\x0b\x32-.yamcs.protobuf.archive.ArchivedParameterInfo\x12\x45\n\x08segments\x18\x02 \x03(\x0b\x32\x33.yamcs.protobuf.archive.ArchiveParameterSegmentInfo\"A\n GetArchivedParameterGroupRequest\x12\x10\n\x08instance\x18\x01 \x01(\t\x12\x0b\n\x03gid\x18\x02 \x01(\r\"p\n\x1e\x41rchivedParameterGroupResponse\x12\x0b\n\x03gid\x18\x01 \x01(\r\x12\x41\n\nparameters\x18\x02 \x03(\x0b\x32-.yamcs.protobuf.archive.ArchivedParameterInfo\",\n\x18\x45nableBackfillingRequest\x12\x10\n\x08instance\x18\x01 \x01(\t\"-\n\x19\x44isableBackfillingRequest\x12\x10\n\x08instance\x18\x01 \x01(\t2\xa8\x0c\n\x13ParameterArchiveApi\x12\x90\x01\n\x0cRebuildRange\x12+.yamcs.protobuf.archive.RebuildRangeRequest\x1a\x16.google.protobuf.Empty\";\x8a\x92\x03\x37\x1a\x30/api/archive/{instance}/parameterArchive:rebuild:\x01*H\x01\x12\xf9\x01\n\x19GetArchivedParametersInfo\x12\x38.yamcs.protobuf.archive.GetArchivedParametersInfoRequest\x1a\x36.yamcs.protobuf.archive.ArchivedParametersInfoResponse\"j\x8a\x92\x03\x66\n&/api/parameter-archive/{instance}/pidsZ<\n8/api/archive/{instance}/parameterArchive/info/parameters0\x01\x12\x95\x02\n\x1cGetArchivedParameterSegments\x12;.yamcs.protobuf.archive.GetArchivedParameterSegmentsRequest\x1a\x39.yamcs.protobuf.archive.ArchivedParameterSegmentsResponse\"}\x8a\x92\x03y\n5/api/parameter-archive/{instance}/pids/{pid}/segmentsZ@\n</api/archive/{instance}/parameterArchive/info/segments/{pid}0\x01\x12\x81\x02\n\x19GetArchivedParameterGroup\x12\x38.yamcs.protobuf.archive.GetArchivedParameterGroupRequest\x1a\x36.yamcs.protobuf.archive.ArchivedParameterGroupResponse\"r\x8a\x92\x03n\n,/api/parameter-archive/{instance}/gids/{gid}Z>\n:/api/archive/{instance}/parameterArchive/info/groups/{gid}0\x01\x12~\n\x05Purge\x12$.yamcs.protobuf.archive.PurgeRequest\x1a\x16.google.protobuf.Empty\"7\x8a\x92\x03\x33\x1a./api/archive/{instance}/parameterArchive:purge:\x01*\x12\xa5\x01\n\x12\x44isableBackfilling\x12\x31.yamcs.protobuf.archive.DisableBackfillingRequest\x1a\x16.google.protobuf.Empty\"D\x8a\x92\x03@\x1a;/api/archive/{instance}/parameterArchive:disableBackfilling:\x01*\x12\xa2\x01\n\x11\x45nableBackfilling\x12\x30.yamcs.protobuf.archive.EnableBackfillingRequest\x1a\x16.google.protobuf.Empty\"C\x8a\x92\x03?\x1a:/api/archive/{instance}/parameterArchive:enableBackfilling:\x01*\x12\x92\x01\n\x14SubscribeBackfilling\x12\x33.yamcs.protobuf.archive.SubscribeBackfillingRequest\x1a\x30.yamcs.protobuf.archive.SubscribeBackfillingData\"\x11\xda\x92\x03\r\n\x0b\x62\x61\x63kfilling0\x01\x1a\x04\xd0\x80\x01\x01\x42\x34\n\x12org.yamcs.protobufB\x1cParameterArchiveServiceProtoP\x01'
  ,
  dependencies=[google_dot_protobuf_dot_empty__pb2.DESCRIPTOR,google_dot_protobuf_dot_timestamp__pb2.DESCRIPTOR,yamcs_dot_api_dot_annotations__pb2.DESCRIPTOR,yamcs_dot_protobuf_dot_yamcs__pb2.DESCRIPTOR,])




_SUBSCRIBEBACKFILLINGREQUEST = _descriptor.Descriptor(
  name='SubscribeBackfillingRequest',
  full_name='yamcs.protobuf.archive.SubscribeBackfillingRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='instance', full_name='yamcs.protobuf.archive.SubscribeBackfillingRequest.instance', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=201,
  serialized_end=248,
)


_SUBSCRIBEBACKFILLINGDATA_BACKFILLFINISHEDINFO = _descriptor.Descriptor(
  name='BackfillFinishedInfo',
  full_name='yamcs.protobuf.archive.SubscribeBackfillingData.BackfillFinishedInfo',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='start', full_name='yamcs.protobuf.archive.SubscribeBackfillingData.BackfillFinishedInfo.start', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='stop', full_name='yamcs.protobuf.archive.SubscribeBackfillingData.BackfillFinishedInfo.stop', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='processedParameters', full_name='yamcs.protobuf.archive.SubscribeBackfillingData.BackfillFinishedInfo.processedParameters', index=2,
      number=3, type=4, cpp_type=4, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=369,
  serialized_end=505,
)

_SUBSCRIBEBACKFILLINGDATA = _descriptor.Descriptor(
  name='SubscribeBackfillingData',
  full_name='yamcs.protobuf.archive.SubscribeBackfillingData',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='finished', full_name='yamcs.protobuf.archive.SubscribeBackfillingData.finished', index=0,
      number=1, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[_SUBSCRIBEBACKFILLINGDATA_BACKFILLFINISHEDINFO, ],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=251,
  serialized_end=505,
)


_REBUILDRANGEREQUEST = _descriptor.Descriptor(
  name='RebuildRangeRequest',
  full_name='yamcs.protobuf.archive.RebuildRangeRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='instance', full_name='yamcs.protobuf.archive.RebuildRangeRequest.instance', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='start', full_name='yamcs.protobuf.archive.RebuildRangeRequest.start', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='stop', full_name='yamcs.protobuf.archive.RebuildRangeRequest.stop', index=2,
      number=3, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=507,
  serialized_end=631,
)


_PURGEREQUEST = _descriptor.Descriptor(
  name='PurgeRequest',
  full_name='yamcs.protobuf.archive.PurgeRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='instance', full_name='yamcs.protobuf.archive.PurgeRequest.instance', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=633,
  serialized_end=665,
)


_GETARCHIVEDPARAMETERSINFOREQUEST = _descriptor.Descriptor(
  name='GetArchivedParametersInfoRequest',
  full_name='yamcs.protobuf.archive.GetArchivedParametersInfoRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='instance', full_name='yamcs.protobuf.archive.GetArchivedParametersInfoRequest.instance', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='filter', full_name='yamcs.protobuf.archive.GetArchivedParametersInfoRequest.filter', index=1,
      number=4, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='limit', full_name='yamcs.protobuf.archive.GetArchivedParametersInfoRequest.limit', index=2,
      number=7, type=5, cpp_type=1, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='next', full_name='yamcs.protobuf.archive.GetArchivedParametersInfoRequest.next', index=3,
      number=8, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=667,
  serialized_end=764,
)


_GETARCHIVEDPARAMETERSEGMENTSREQUEST = _descriptor.Descriptor(
  name='GetArchivedParameterSegmentsRequest',
  full_name='yamcs.protobuf.archive.GetArchivedParameterSegmentsRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='instance', full_name='yamcs.protobuf.archive.GetArchivedParameterSegmentsRequest.instance', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='pid', full_name='yamcs.protobuf.archive.GetArchivedParameterSegmentsRequest.pid', index=1,
      number=2, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='start', full_name='yamcs.protobuf.archive.GetArchivedParameterSegmentsRequest.start', index=2,
      number=3, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='stop', full_name='yamcs.protobuf.archive.GetArchivedParameterSegmentsRequest.stop', index=3,
      number=4, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=767,
  serialized_end=920,
)


_ARCHIVEDPARAMETERINFO = _descriptor.Descriptor(
  name='ArchivedParameterInfo',
  full_name='yamcs.protobuf.archive.ArchivedParameterInfo',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='pid', full_name='yamcs.protobuf.archive.ArchivedParameterInfo.pid', index=0,
      number=1, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='parameter', full_name='yamcs.protobuf.archive.ArchivedParameterInfo.parameter', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='rawType', full_name='yamcs.protobuf.archive.ArchivedParameterInfo.rawType', index=2,
      number=3, type=14, cpp_type=8, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='engType', full_name='yamcs.protobuf.archive.ArchivedParameterInfo.engType', index=3,
      number=4, type=14, cpp_type=8, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='gids', full_name='yamcs.protobuf.archive.ArchivedParameterInfo.gids', index=4,
      number=5, type=13, cpp_type=3, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=923,
  serialized_end=1082,
)


_ARCHIVEDPARAMETERSINFORESPONSE = _descriptor.Descriptor(
  name='ArchivedParametersInfoResponse',
  full_name='yamcs.protobuf.archive.ArchivedParametersInfoResponse',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='pids', full_name='yamcs.protobuf.archive.ArchivedParametersInfoResponse.pids', index=0,
      number=1, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='continuationToken', full_name='yamcs.protobuf.archive.ArchivedParametersInfoResponse.continuationToken', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1084,
  serialized_end=1204,
)


_ARCHIVEPARAMETERSEGMENTINFO = _descriptor.Descriptor(
  name='ArchiveParameterSegmentInfo',
  full_name='yamcs.protobuf.archive.ArchiveParameterSegmentInfo',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='groupId', full_name='yamcs.protobuf.archive.ArchiveParameterSegmentInfo.groupId', index=0,
      number=1, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='start', full_name='yamcs.protobuf.archive.ArchiveParameterSegmentInfo.start', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='end', full_name='yamcs.protobuf.archive.ArchiveParameterSegmentInfo.end', index=2,
      number=3, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='count', full_name='yamcs.protobuf.archive.ArchiveParameterSegmentInfo.count', index=3,
      number=4, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1207,
  serialized_end=1352,
)


_ARCHIVEDPARAMETERSEGMENTSRESPONSE = _descriptor.Descriptor(
  name='ArchivedParameterSegmentsResponse',
  full_name='yamcs.protobuf.archive.ArchivedParameterSegmentsResponse',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='parameterInfo', full_name='yamcs.protobuf.archive.ArchivedParameterSegmentsResponse.parameterInfo', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='segments', full_name='yamcs.protobuf.archive.ArchivedParameterSegmentsResponse.segments', index=1,
      number=2, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1355,
  serialized_end=1531,
)


_GETARCHIVEDPARAMETERGROUPREQUEST = _descriptor.Descriptor(
  name='GetArchivedParameterGroupRequest',
  full_name='yamcs.protobuf.archive.GetArchivedParameterGroupRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='instance', full_name='yamcs.protobuf.archive.GetArchivedParameterGroupRequest.instance', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='gid', full_name='yamcs.protobuf.archive.GetArchivedParameterGroupRequest.gid', index=1,
      number=2, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1533,
  serialized_end=1598,
)


_ARCHIVEDPARAMETERGROUPRESPONSE = _descriptor.Descriptor(
  name='ArchivedParameterGroupResponse',
  full_name='yamcs.protobuf.archive.ArchivedParameterGroupResponse',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='gid', full_name='yamcs.protobuf.archive.ArchivedParameterGroupResponse.gid', index=0,
      number=1, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='parameters', full_name='yamcs.protobuf.archive.ArchivedParameterGroupResponse.parameters', index=1,
      number=2, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1600,
  serialized_end=1712,
)


_ENABLEBACKFILLINGREQUEST = _descriptor.Descriptor(
  name='EnableBackfillingRequest',
  full_name='yamcs.protobuf.archive.EnableBackfillingRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='instance', full_name='yamcs.protobuf.archive.EnableBackfillingRequest.instance', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1714,
  serialized_end=1758,
)


_DISABLEBACKFILLINGREQUEST = _descriptor.Descriptor(
  name='DisableBackfillingRequest',
  full_name='yamcs.protobuf.archive.DisableBackfillingRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='instance', full_name='yamcs.protobuf.archive.DisableBackfillingRequest.instance', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1760,
  serialized_end=1805,
)

_SUBSCRIBEBACKFILLINGDATA_BACKFILLFINISHEDINFO.fields_by_name['start'].message_type = google_dot_protobuf_dot_timestamp__pb2._TIMESTAMP
_SUBSCRIBEBACKFILLINGDATA_BACKFILLFINISHEDINFO.fields_by_name['stop'].message_type = google_dot_protobuf_dot_timestamp__pb2._TIMESTAMP
_SUBSCRIBEBACKFILLINGDATA_BACKFILLFINISHEDINFO.containing_type = _SUBSCRIBEBACKFILLINGDATA
_SUBSCRIBEBACKFILLINGDATA.fields_by_name['finished'].message_type = _SUBSCRIBEBACKFILLINGDATA_BACKFILLFINISHEDINFO
_REBUILDRANGEREQUEST.fields_by_name['start'].message_type = google_dot_protobuf_dot_timestamp__pb2._TIMESTAMP
_REBUILDRANGEREQUEST.fields_by_name['stop'].message_type = google_dot_protobuf_dot_timestamp__pb2._TIMESTAMP
_GETARCHIVEDPARAMETERSEGMENTSREQUEST.fields_by_name['start'].message_type = google_dot_protobuf_dot_timestamp__pb2._TIMESTAMP
_GETARCHIVEDPARAMETERSEGMENTSREQUEST.fields_by_name['stop'].message_type = google_dot_protobuf_dot_timestamp__pb2._TIMESTAMP
_ARCHIVEDPARAMETERINFO.fields_by_name['rawType'].enum_type = yamcs_dot_protobuf_dot_yamcs__pb2._VALUE_TYPE
_ARCHIVEDPARAMETERINFO.fields_by_name['engType'].enum_type = yamcs_dot_protobuf_dot_yamcs__pb2._VALUE_TYPE
_ARCHIVEDPARAMETERSINFORESPONSE.fields_by_name['pids'].message_type = _ARCHIVEDPARAMETERINFO
_ARCHIVEPARAMETERSEGMENTINFO.fields_by_name['start'].message_type = google_dot_protobuf_dot_timestamp__pb2._TIMESTAMP
_ARCHIVEPARAMETERSEGMENTINFO.fields_by_name['end'].message_type = google_dot_protobuf_dot_timestamp__pb2._TIMESTAMP
_ARCHIVEDPARAMETERSEGMENTSRESPONSE.fields_by_name['parameterInfo'].message_type = _ARCHIVEDPARAMETERINFO
_ARCHIVEDPARAMETERSEGMENTSRESPONSE.fields_by_name['segments'].message_type = _ARCHIVEPARAMETERSEGMENTINFO
_ARCHIVEDPARAMETERGROUPRESPONSE.fields_by_name['parameters'].message_type = _ARCHIVEDPARAMETERINFO
DESCRIPTOR.message_types_by_name['SubscribeBackfillingRequest'] = _SUBSCRIBEBACKFILLINGREQUEST
DESCRIPTOR.message_types_by_name['SubscribeBackfillingData'] = _SUBSCRIBEBACKFILLINGDATA
DESCRIPTOR.message_types_by_name['RebuildRangeRequest'] = _REBUILDRANGEREQUEST
DESCRIPTOR.message_types_by_name['PurgeRequest'] = _PURGEREQUEST
DESCRIPTOR.message_types_by_name['GetArchivedParametersInfoRequest'] = _GETARCHIVEDPARAMETERSINFOREQUEST
DESCRIPTOR.message_types_by_name['GetArchivedParameterSegmentsRequest'] = _GETARCHIVEDPARAMETERSEGMENTSREQUEST
DESCRIPTOR.message_types_by_name['ArchivedParameterInfo'] = _ARCHIVEDPARAMETERINFO
DESCRIPTOR.message_types_by_name['ArchivedParametersInfoResponse'] = _ARCHIVEDPARAMETERSINFORESPONSE
DESCRIPTOR.message_types_by_name['ArchiveParameterSegmentInfo'] = _ARCHIVEPARAMETERSEGMENTINFO
DESCRIPTOR.message_types_by_name['ArchivedParameterSegmentsResponse'] = _ARCHIVEDPARAMETERSEGMENTSRESPONSE
DESCRIPTOR.message_types_by_name['GetArchivedParameterGroupRequest'] = _GETARCHIVEDPARAMETERGROUPREQUEST
DESCRIPTOR.message_types_by_name['ArchivedParameterGroupResponse'] = _ARCHIVEDPARAMETERGROUPRESPONSE
DESCRIPTOR.message_types_by_name['EnableBackfillingRequest'] = _ENABLEBACKFILLINGREQUEST
DESCRIPTOR.message_types_by_name['DisableBackfillingRequest'] = _DISABLEBACKFILLINGREQUEST
_sym_db.RegisterFileDescriptor(DESCRIPTOR)

SubscribeBackfillingRequest = _reflection.GeneratedProtocolMessageType('SubscribeBackfillingRequest', (_message.Message,), {
  'DESCRIPTOR' : _SUBSCRIBEBACKFILLINGREQUEST,
  '__module__' : 'yamcs.protobuf.archive.parameter_archive_service_pb2'
  # @@protoc_insertion_point(class_scope:yamcs.protobuf.archive.SubscribeBackfillingRequest)
  })
_sym_db.RegisterMessage(SubscribeBackfillingRequest)

SubscribeBackfillingData = _reflection.GeneratedProtocolMessageType('SubscribeBackfillingData', (_message.Message,), {

  'BackfillFinishedInfo' : _reflection.GeneratedProtocolMessageType('BackfillFinishedInfo', (_message.Message,), {
    'DESCRIPTOR' : _SUBSCRIBEBACKFILLINGDATA_BACKFILLFINISHEDINFO,
    '__module__' : 'yamcs.protobuf.archive.parameter_archive_service_pb2'
    # @@protoc_insertion_point(class_scope:yamcs.protobuf.archive.SubscribeBackfillingData.BackfillFinishedInfo)
    })
  ,
  'DESCRIPTOR' : _SUBSCRIBEBACKFILLINGDATA,
  '__module__' : 'yamcs.protobuf.archive.parameter_archive_service_pb2'
  # @@protoc_insertion_point(class_scope:yamcs.protobuf.archive.SubscribeBackfillingData)
  })
_sym_db.RegisterMessage(SubscribeBackfillingData)
_sym_db.RegisterMessage(SubscribeBackfillingData.BackfillFinishedInfo)

RebuildRangeRequest = _reflection.GeneratedProtocolMessageType('RebuildRangeRequest', (_message.Message,), {
  'DESCRIPTOR' : _REBUILDRANGEREQUEST,
  '__module__' : 'yamcs.protobuf.archive.parameter_archive_service_pb2'
  # @@protoc_insertion_point(class_scope:yamcs.protobuf.archive.RebuildRangeRequest)
  })
_sym_db.RegisterMessage(RebuildRangeRequest)

PurgeRequest = _reflection.GeneratedProtocolMessageType('PurgeRequest', (_message.Message,), {
  'DESCRIPTOR' : _PURGEREQUEST,
  '__module__' : 'yamcs.protobuf.archive.parameter_archive_service_pb2'
  # @@protoc_insertion_point(class_scope:yamcs.protobuf.archive.PurgeRequest)
  })
_sym_db.RegisterMessage(PurgeRequest)

GetArchivedParametersInfoRequest = _reflection.GeneratedProtocolMessageType('GetArchivedParametersInfoRequest', (_message.Message,), {
  'DESCRIPTOR' : _GETARCHIVEDPARAMETERSINFOREQUEST,
  '__module__' : 'yamcs.protobuf.archive.parameter_archive_service_pb2'
  # @@protoc_insertion_point(class_scope:yamcs.protobuf.archive.GetArchivedParametersInfoRequest)
  })
_sym_db.RegisterMessage(GetArchivedParametersInfoRequest)

GetArchivedParameterSegmentsRequest = _reflection.GeneratedProtocolMessageType('GetArchivedParameterSegmentsRequest', (_message.Message,), {
  'DESCRIPTOR' : _GETARCHIVEDPARAMETERSEGMENTSREQUEST,
  '__module__' : 'yamcs.protobuf.archive.parameter_archive_service_pb2'
  # @@protoc_insertion_point(class_scope:yamcs.protobuf.archive.GetArchivedParameterSegmentsRequest)
  })
_sym_db.RegisterMessage(GetArchivedParameterSegmentsRequest)

ArchivedParameterInfo = _reflection.GeneratedProtocolMessageType('ArchivedParameterInfo', (_message.Message,), {
  'DESCRIPTOR' : _ARCHIVEDPARAMETERINFO,
  '__module__' : 'yamcs.protobuf.archive.parameter_archive_service_pb2'
  # @@protoc_insertion_point(class_scope:yamcs.protobuf.archive.ArchivedParameterInfo)
  })
_sym_db.RegisterMessage(ArchivedParameterInfo)

ArchivedParametersInfoResponse = _reflection.GeneratedProtocolMessageType('ArchivedParametersInfoResponse', (_message.Message,), {
  'DESCRIPTOR' : _ARCHIVEDPARAMETERSINFORESPONSE,
  '__module__' : 'yamcs.protobuf.archive.parameter_archive_service_pb2'
  # @@protoc_insertion_point(class_scope:yamcs.protobuf.archive.ArchivedParametersInfoResponse)
  })
_sym_db.RegisterMessage(ArchivedParametersInfoResponse)

ArchiveParameterSegmentInfo = _reflection.GeneratedProtocolMessageType('ArchiveParameterSegmentInfo', (_message.Message,), {
  'DESCRIPTOR' : _ARCHIVEPARAMETERSEGMENTINFO,
  '__module__' : 'yamcs.protobuf.archive.parameter_archive_service_pb2'
  # @@protoc_insertion_point(class_scope:yamcs.protobuf.archive.ArchiveParameterSegmentInfo)
  })
_sym_db.RegisterMessage(ArchiveParameterSegmentInfo)

ArchivedParameterSegmentsResponse = _reflection.GeneratedProtocolMessageType('ArchivedParameterSegmentsResponse', (_message.Message,), {
  'DESCRIPTOR' : _ARCHIVEDPARAMETERSEGMENTSRESPONSE,
  '__module__' : 'yamcs.protobuf.archive.parameter_archive_service_pb2'
  # @@protoc_insertion_point(class_scope:yamcs.protobuf.archive.ArchivedParameterSegmentsResponse)
  })
_sym_db.RegisterMessage(ArchivedParameterSegmentsResponse)

GetArchivedParameterGroupRequest = _reflection.GeneratedProtocolMessageType('GetArchivedParameterGroupRequest', (_message.Message,), {
  'DESCRIPTOR' : _GETARCHIVEDPARAMETERGROUPREQUEST,
  '__module__' : 'yamcs.protobuf.archive.parameter_archive_service_pb2'
  # @@protoc_insertion_point(class_scope:yamcs.protobuf.archive.GetArchivedParameterGroupRequest)
  })
_sym_db.RegisterMessage(GetArchivedParameterGroupRequest)

ArchivedParameterGroupResponse = _reflection.GeneratedProtocolMessageType('ArchivedParameterGroupResponse', (_message.Message,), {
  'DESCRIPTOR' : _ARCHIVEDPARAMETERGROUPRESPONSE,
  '__module__' : 'yamcs.protobuf.archive.parameter_archive_service_pb2'
  # @@protoc_insertion_point(class_scope:yamcs.protobuf.archive.ArchivedParameterGroupResponse)
  })
_sym_db.RegisterMessage(ArchivedParameterGroupResponse)

EnableBackfillingRequest = _reflection.GeneratedProtocolMessageType('EnableBackfillingRequest', (_message.Message,), {
  'DESCRIPTOR' : _ENABLEBACKFILLINGREQUEST,
  '__module__' : 'yamcs.protobuf.archive.parameter_archive_service_pb2'
  # @@protoc_insertion_point(class_scope:yamcs.protobuf.archive.EnableBackfillingRequest)
  })
_sym_db.RegisterMessage(EnableBackfillingRequest)

DisableBackfillingRequest = _reflection.GeneratedProtocolMessageType('DisableBackfillingRequest', (_message.Message,), {
  'DESCRIPTOR' : _DISABLEBACKFILLINGREQUEST,
  '__module__' : 'yamcs.protobuf.archive.parameter_archive_service_pb2'
  # @@protoc_insertion_point(class_scope:yamcs.protobuf.archive.DisableBackfillingRequest)
  })
_sym_db.RegisterMessage(DisableBackfillingRequest)


DESCRIPTOR._options = None

_PARAMETERARCHIVEAPI = _descriptor.ServiceDescriptor(
  name='ParameterArchiveApi',
  full_name='yamcs.protobuf.archive.ParameterArchiveApi',
  file=DESCRIPTOR,
  index=0,
  serialized_options=b'\320\200\001\001',
  serialized_start=1808,
  serialized_end=3384,
  methods=[
  _descriptor.MethodDescriptor(
    name='RebuildRange',
    full_name='yamcs.protobuf.archive.ParameterArchiveApi.RebuildRange',
    index=0,
    containing_service=None,
    input_type=_REBUILDRANGEREQUEST,
    output_type=google_dot_protobuf_dot_empty__pb2._EMPTY,
    serialized_options=b'\212\222\0037\0320/api/archive/{instance}/parameterArchive:rebuild:\001*H\001',
  ),
  _descriptor.MethodDescriptor(
    name='GetArchivedParametersInfo',
    full_name='yamcs.protobuf.archive.ParameterArchiveApi.GetArchivedParametersInfo',
    index=1,
    containing_service=None,
    input_type=_GETARCHIVEDPARAMETERSINFOREQUEST,
    output_type=_ARCHIVEDPARAMETERSINFORESPONSE,
    serialized_options=b'\212\222\003f\n&/api/parameter-archive/{instance}/pidsZ<\n8/api/archive/{instance}/parameterArchive/info/parameters0\001',
  ),
  _descriptor.MethodDescriptor(
    name='GetArchivedParameterSegments',
    full_name='yamcs.protobuf.archive.ParameterArchiveApi.GetArchivedParameterSegments',
    index=2,
    containing_service=None,
    input_type=_GETARCHIVEDPARAMETERSEGMENTSREQUEST,
    output_type=_ARCHIVEDPARAMETERSEGMENTSRESPONSE,
    serialized_options=b'\212\222\003y\n5/api/parameter-archive/{instance}/pids/{pid}/segmentsZ@\n</api/archive/{instance}/parameterArchive/info/segments/{pid}0\001',
  ),
  _descriptor.MethodDescriptor(
    name='GetArchivedParameterGroup',
    full_name='yamcs.protobuf.archive.ParameterArchiveApi.GetArchivedParameterGroup',
    index=3,
    containing_service=None,
    input_type=_GETARCHIVEDPARAMETERGROUPREQUEST,
    output_type=_ARCHIVEDPARAMETERGROUPRESPONSE,
    serialized_options=b'\212\222\003n\n,/api/parameter-archive/{instance}/gids/{gid}Z>\n:/api/archive/{instance}/parameterArchive/info/groups/{gid}0\001',
  ),
  _descriptor.MethodDescriptor(
    name='Purge',
    full_name='yamcs.protobuf.archive.ParameterArchiveApi.Purge',
    index=4,
    containing_service=None,
    input_type=_PURGEREQUEST,
    output_type=google_dot_protobuf_dot_empty__pb2._EMPTY,
    serialized_options=b'\212\222\0033\032./api/archive/{instance}/parameterArchive:purge:\001*',
  ),
  _descriptor.MethodDescriptor(
    name='DisableBackfilling',
    full_name='yamcs.protobuf.archive.ParameterArchiveApi.DisableBackfilling',
    index=5,
    containing_service=None,
    input_type=_DISABLEBACKFILLINGREQUEST,
    output_type=google_dot_protobuf_dot_empty__pb2._EMPTY,
    serialized_options=b'\212\222\003@\032;/api/archive/{instance}/parameterArchive:disableBackfilling:\001*',
  ),
  _descriptor.MethodDescriptor(
    name='EnableBackfilling',
    full_name='yamcs.protobuf.archive.ParameterArchiveApi.EnableBackfilling',
    index=6,
    containing_service=None,
    input_type=_ENABLEBACKFILLINGREQUEST,
    output_type=google_dot_protobuf_dot_empty__pb2._EMPTY,
    serialized_options=b'\212\222\003?\032:/api/archive/{instance}/parameterArchive:enableBackfilling:\001*',
  ),
  _descriptor.MethodDescriptor(
    name='SubscribeBackfilling',
    full_name='yamcs.protobuf.archive.ParameterArchiveApi.SubscribeBackfilling',
    index=7,
    containing_service=None,
    input_type=_SUBSCRIBEBACKFILLINGREQUEST,
    output_type=_SUBSCRIBEBACKFILLINGDATA,
    serialized_options=b'\332\222\003\r\n\013backfilling',
  ),
])
_sym_db.RegisterServiceDescriptor(_PARAMETERARCHIVEAPI)

DESCRIPTOR.services_by_name['ParameterArchiveApi'] = _PARAMETERARCHIVEAPI

# @@protoc_insertion_point(module_scope)



from __future__ import annotations

import datetime
import functools
import os
import urllib.parse
from typing import (
    TYPE_CHECKING,
    Any,
    Callable,
    Iterable,
    Iterator,
    List,
    Mapping,
    Optional,
    Union,
)

from google.protobuf import timestamp_pb2
from google.protobuf.message import DecodeError
from yamcs.client.core.auth import APIKeyCredentials, Credentials
from yamcs.client.core.context import Context
from yamcs.client.core.exceptions import YamcsError
from yamcs.client.core.futures import WebSocketSubscriptionFuture
from yamcs.client.core.helpers import (
    delimit_protobuf,
    do_get,
    parse_server_time,
    to_server_time,
)
from yamcs.client.core.subscriptions import WebSocketSubscriptionManager
from yamcs.client.model import (
    AuthInfo,
    Event,
    Instance,
    InstanceTemplate,
    Link,
    LoadParameterValuesResult,
    Processor,
    RdbTablespace,
    ServerInfo,
    Service,
    UserInfo,
)
from yamcs.client.tmtc.client import _build_value_proto
from yamcs.protobuf.archive import rocksdb_service_pb2
from yamcs.protobuf.auth import auth_pb2
from yamcs.protobuf.events import events_pb2, events_service_pb2
from yamcs.protobuf.iam import iam_pb2
from yamcs.protobuf.instances import instances_pb2, instances_service_pb2
from yamcs.protobuf.links import links_pb2
from yamcs.protobuf.processing import processing_pb2
from yamcs.protobuf.pvalue import pvalue_service_pb2
from yamcs.protobuf.server import server_service_pb2
from yamcs.protobuf.services import services_service_pb2
from yamcs.protobuf.time import time_service_pb2

if TYPE_CHECKING:
    from yamcs.client.archive.client import ArchiveClient
    from yamcs.client.filetransfer.client import FileTransferClient
    from yamcs.client.links.client import LinkClient
    from yamcs.client.mdb.client import MDBClient
    from yamcs.client.storage.client import StorageClient
    from yamcs.client.tco.client import TCOClient
    from yamcs.client.timeline.client import TimelineClient
    from yamcs.client.tmtc.client import ProcessorClient
    from yamcs.client.tmtc.model import ValueUpdate

__all__ = [
    "GLOBAL_INSTANCE",
    "LinkSubscription",
    "TimeSubscription",
    "YamcsClient",
]

GLOBAL_INSTANCE = "_global"


def _wrap_callback_parse_time_info(subscription, on_data, message):
    """
    Wraps a user callback to parse TimeInfo
    from a WebSocket data message
    """
    pb = timestamp_pb2.Timestamp()
    message.Unpack(pb)
    time = parse_server_time(pb)
    subscription._process(time)
    if on_data:
        on_data(time)


def _wrap_callback_parse_event(on_data, message):
    """
    Wraps a user callback to parse Events
    from a WebSocket data message
    """
    pb = events_pb2.Event()
    message.Unpack(pb)
    event = Event(pb)
    on_data(event)


def _wrap_callback_parse_link_event(subscription, on_data, message):
    """
    Wraps a user callback to parse Link[] updates
    from a WebSocket data message
    """
    pb = links_pb2.LinkEvent()
    message.Unpack(pb)
    links = [Link(link_pb) for link_pb in pb.links]
    subscription._process(links)
    if on_data:
        on_data(links)


class TimeSubscription(WebSocketSubscriptionFuture):
    """
    Local object providing access to time updates.

    A subscription object stores the last time info.
    """

    def __init__(self, manager):
        super(TimeSubscription, self).__init__(manager)

        self.time: Optional[datetime.datetime] = None
        """The last time info."""

    def _process(self, time):
        self.time = time


class LinkSubscription(WebSocketSubscriptionFuture):
    """
    Local object providing access to data link updates.

    A subscription object stores the last link info for
    each link.
    """

    def __init__(self, manager):
        super(LinkSubscription, self).__init__(manager)

        self._cache = {}
        """Link cache keyed by name."""

    def get_link(self, name: str) -> Optional[Link]:
        """
        Returns the latest link state.

        :param name:
            Identifying name of the data link
        """
        if name in self._cache:
            return self._cache[name]
        return None

    def list_links(self) -> List[Link]:
        """
        Returns a snapshot of all links.
        """
        return [self._cache[k] for k in self._cache]

    def _process(self, links: List[Link]):
        for link in links:
            self._cache[link.name] = link


class YamcsClient:
    """
    Client for accessing core Yamcs resources.
    """

    def __init__(
        self,
        address: str,
        *,
        tls: bool = False,
        tls_verify: Union[bool, str] = True,
        credentials: Optional[Credentials] = None,
        user_agent: Optional[str] = None,
        keep_alive: bool = True,
        on_token_update: Optional[Callable[[Credentials], None]] = None,
    ):
        """
        :param address:
            The address of Yamcs in the format 'hostname:port'
        :param tls:
            Whether TLS encryption is expected
        :param tls_verify:
            Whether server certificate verification is
            enabled (only applicable if ``tls=True``).
            As an alternative to a boolean value, this option
            may be set to a path containing the appropriate
            TLS CA certificate bundle.
        :param credentials:
            Credentials for when the server is secured
        :param user_agent:
            Optionally override the default user agent
        :param keep_alive:
            Automatically renew the client session. If disabled,
            the session will terminate after about 30 minutes of
            inactivity.

            This property is only considered when accessing a
            server that requires authentication.
        """
        self.ctx = Context(
            address=address,
            tls=tls,
            credentials=credentials,
            user_agent=user_agent,
            on_token_update=on_token_update,
            tls_verify=tls_verify,
            keep_alive=keep_alive,
        )

    @staticmethod
    def from_environment():
        """
        Create a :class:`.YamcsClient`, initialized from environment variables.

        This recognizes the following environment variables:

        ``YAMCS_URL``
            Yamcs server URL

        ``YAMCS_API_KEY``
            Yamcs API key (currently only assigned to script activities)
        """
        url = os.environ["YAMCS_URL"]

        credentials = None
        api_key = os.environ.get("YAMCS_API_KEY")
        if api_key:
            credentials = APIKeyCredentials(api_key)

        return YamcsClient(url, credentials=credentials)

    def __enter__(self):
        return self

    def __exit__(self, exc_type, exc_value, traceback):
        self.close()

    def get_time(self, instance) -> Optional[datetime.datetime]:
        """
        Return the current mission time for the specified instance.
        """
        url = f"/instances/{instance}"
        response = self.ctx.get_proto(url)
        message = instances_pb2.YamcsInstance()
        message.ParseFromString(response.content)
        if message.HasField("missionTime"):
            return parse_server_time(message.missionTime)
        return None

    def get_server_info(self) -> ServerInfo:
        """
        Return general server info.
        """
        response = self.ctx.get_proto(path="")
        message = server_service_pb2.GetServerInfoResponse()
        message.ParseFromString(response.content)
        return ServerInfo(message)

    def get_auth_info(self) -> AuthInfo:
        """
        Returns general authentication information. This operation
        does not require authenticating and is useful to test
        if a server requires authentication or not.
        """
        response = do_get(
            self.ctx.session,
            self.ctx.auth_root,  # Full URL, so don't use get_proto
            headers={"Accept": "application/protobuf"},
        )
        if 200 <= response.status_code < 300:
            try:
                message = auth_pb2.AuthInfo()
                message.ParseFromString(response.content)
                return AuthInfo(message)
            except DecodeError:
                raise YamcsError(
                    f"Unexpected response. Verify Yamcs URL: {self.ctx.url}"
                )
        else:
            raise YamcsError(f"{response.status_code} Server Error")

    def get_user_info(self) -> UserInfo:
        """
        Get information on the authenticated user.
        """
        response = self.ctx.get_proto(path="/user")
        message = iam_pb2.UserInfo()
        message.ParseFromString(response.content)
        return UserInfo(message)

    def get_mdb(self, instance: str) -> "MDBClient":
        """
        Return an object for working with the MDB of the specified instance.

        :param instance:
            A Yamcs instance name.
        """
        from yamcs.client.mdb.client import MDBClient

        return MDBClient(self.ctx, instance)

    def get_archive(self, instance: str) -> "ArchiveClient":
        """
        Return an object for working with the Archive of the specified instance.

        :param instance:
            A Yamcs instance name.
        """
        from yamcs.client.archive.client import ArchiveClient

        return ArchiveClient(self.ctx, instance)

    def get_file_transfer_client(self, instance: str) -> "FileTransferClient":
        """
        Return an object for working with file transfers on a specified instance.

        :param instance:
            A Yamcs instance name.
        """
        from yamcs.client.filetransfer.client import FileTransferClient

        return FileTransferClient(self.ctx, instance)

    def get_tco_client(self, instance: str, service: str) -> "TCOClient":
        """
        Return an object for Time Correlation API calls on a specified service.

        :param instance:
            A Yamcs instance name.
        :param service:
            Target service name.
        """
        from yamcs.client.tco.client import TCOClient

        return TCOClient(self.ctx, instance, service)

    def get_storage_client(self) -> "StorageClient":
        """
        Return an object for working with object storage
        """
        from yamcs.client.storage.client import StorageClient

        return StorageClient(self.ctx)

    def get_timeline_client(self, instance: str) -> "TimelineClient":
        """
        Return an object for working with Yamcs timeline items

        :param instance:
            A Yamcs instance name.
        """
        from yamcs.client.timeline.client import TimelineClient

        return TimelineClient(self.ctx, instance)

    def create_instance(
        self,
        name: str,
        template: str,
        args: Optional[Mapping[str, Any]] = None,
        labels: Optional[Mapping[str, str]] = None,
    ):
        """
        Create a new instance based on an existing template. This method blocks
        until the instance is fully started.

        :param instance:
            A Yamcs instance name.
        :param template:
            The name of an existing template.
        """
        req = instances_service_pb2.CreateInstanceRequest()
        req.name = name
        req.template = template
        if args:
            for k in args:
                req.templateArgs[k] = args[k]
        if labels:
            for k in labels:
                req.labels[k] = labels[k]
        url = "/instances"
        self.ctx.post_proto(url, data=req.SerializeToString())

    def list_instance_templates(self) -> Iterable[InstanceTemplate]:
        """
        List the available instance templates.
        """
        response = self.ctx.get_proto(path="/instance-templates")
        message = instances_service_pb2.ListInstanceTemplatesResponse()
        message.ParseFromString(response.content)
        templates = getattr(message, "templates")
        return iter([InstanceTemplate(template) for template in templates])

    def list_services(self, instance: str) -> Iterable[Service]:
        """
        List the services for an instance.

        :param instance:
            A Yamcs instance name.
        """
        # Server does not do pagination on listings of this resource.
        # Return an iterator anyway for similarity with other API methods
        url = f"/services/{instance}"
        response = self.ctx.get_proto(path=url)
        message = services_service_pb2.ListServicesResponse()
        message.ParseFromString(response.content)
        services = getattr(message, "services")
        return iter([Service(service) for service in services])

    def start_service(self, instance: str, service: str):
        """
        Starts a single service.

        :param instance:
            A Yamcs instance name.
        :param service:
            The name of the service.
        """
        url = f"/services/{instance}/{service}:start"
        self.ctx.post_proto(url)

    def stop_service(self, instance: str, service: str):
        """
        Stops a single service.

        :param instance:
            A Yamcs instance name.
        :param service:
            The name of the service.
        """
        url = f"/services/{instance}/{service}:stop"
        self.ctx.post_proto(url)

    def list_processors(self, instance: Optional[str] = None) -> Iterable[Processor]:
        """
        Lists the processors.

        Processors are returned in lexicographical order.

        :param instance:
            A Yamcs instance name.
        """
        # Server does not do pagination on listings of this resource.
        # Return an iterator anyway for similarity with other API methods
        url = "/processors"
        if instance:
            url += "?instance=" + instance
        response = self.ctx.get_proto(path=url)
        message = processing_pb2.ListProcessorsResponse()
        message.ParseFromString(response.content)
        processors = getattr(message, "processors")
        return iter([Processor(processor) for processor in processors])

    def get_processor(self, instance: str, processor: str) -> "ProcessorClient":
        """
        Return an object for working with a specific Yamcs processor.

        :param instance:
            A Yamcs instance name.
        :param processor:
            A processor name within that instance.
        """
        from yamcs.client.tmtc.client import ProcessorClient

        return ProcessorClient(self.ctx, instance, processor)

    def delete_processor(self, instance: str, processor: str):
        """
        Delete a processor.

        :param instance:
            A Yamcs instance name.
        :param processor:
            A processor name within that instance.
        """
        url = f"/processors/{instance}/{processor}"
        self.ctx.delete_proto(url)

    def get_link(self, instance: str, link: str) -> "LinkClient":
        """
        Return an object for working with a specific Yamcs link.

        :param instance:
            A Yamcs instance name.
        :param link:
            A link name within that instance.
        """
        from yamcs.client.links.client import LinkClient

        return LinkClient(self.ctx, instance, link)

    def list_instances(self) -> Iterable[Instance]:
        """
        Lists the instances.

        Instances are returned in lexicographical order.
        """
        # Server does not do pagination on listings of this resource.
        # Return an iterator anyway for similarity with other API methods
        response = self.ctx.get_proto(path="/instances")
        message = instances_service_pb2.ListInstancesResponse()
        message.ParseFromString(response.content)
        instances = getattr(message, "instances")
        return iter([Instance(instance) for instance in instances])

    def start_instance(self, instance: str):
        """
        Starts a single instance.

        :param instance:
            A Yamcs instance name.
        """
        url = f"/instances/{instance}:start"
        self.ctx.post_proto(url)

    def stop_instance(self, instance: str):
        """
        Stops a single instance.

        :param instance:
            A Yamcs instance name.
        """
        url = f"/instances/{instance}:stop"
        self.ctx.post_proto(url)

    def restart_instance(self, instance: str):
        """
        Restarts a single instance.

        :param instance:
            A Yamcs instance name.
        """
        url = f"/instances/{instance}:restart"
        self.ctx.post_proto(url)

    def list_links(self, instance: str) -> Iterable[Link]:
        """
        Lists the data links visible to this client.

        Data links are returned in random order.

        :param instance:
            A Yamcs instance name.
        """
        # Server does not do pagination on listings of this resource.
        # Return an iterator anyway for similarity with other API methods
        response = self.ctx.get_proto(path="/links/" + instance)
        message = links_pb2.ListLinksResponse()
        message.ParseFromString(response.content)
        links = getattr(message, "links")
        return iter([Link(link) for link in links])

    def list_rdb_tablespaces(self) -> Iterable[RdbTablespace]:
        """
        Lists RocksDB tablespaces.
        """
        response = self.ctx.get_proto(path="/archive/rocksdb/tablespaces")
        message = rocksdb_service_pb2.ListRocksDbTablespacesResponse()
        message.ParseFromString(response.content)
        tablespaces = getattr(message, "tablespaces")
        return iter([RdbTablespace(tablespace) for tablespace in tablespaces])

    def compact_rdb_column_family(
        self,
        tablespace: str,
        cf: str,
        dbpath: Optional[str] = None,
    ):
        """
        Compact a RocksDB column family.

        :param tablespace:
            RocksDB tablespace name
        :param cf:
            Column family name
        :param dbpath:
            Optional path under the tablespace root.
        """
        req = rocksdb_service_pb2.CompactDatabaseRequest()
        req.cfname = cf
        encoded_name = urllib.parse.quote_plus(dbpath or "")
        url = f"/archive/rocksdb/{tablespace}/{encoded_name}:compact"
        self.ctx.post_proto(url, data=req.SerializeToString())

    def send_event(
        self,
        instance: str,
        message: str,
        event_type: Optional[str] = None,
        time: Optional[datetime.datetime] = None,
        severity: Optional[str] = "info",
        source: Optional[str] = None,
        sequence_number: Optional[int] = None,
        extra: Optional[Mapping[str, str]] = None,
    ):
        """
        Post a new event.

        :param instance:
            A Yamcs instance name.
        :param message:
            Event message.
        :param event_type:
            Type of event.
        :param severity:
            The severity level of the event. One of ``info``,
            ``watch``, ``warning``, ``distress``, ``critical``
            or ``severe``. Defaults to ``info``.
        :param time:
            Time of the event. If unspecified, defaults to mission time.
        :param source:
            Source of the event. Useful for grouping events in the
            archive. When unset this defaults to ``User``.
        :param event_type:
            Event type.
        :param sequence_number:
            Sequence number of this event. This is used to
            determine unicity of events at the same time and
            coming from the same source. If not set Yamcs
            will automatically assign a sequential number as
            if every submitted event is unique.
        :param extra:
            Extra event properties.
        """
        req = events_service_pb2.CreateEventRequest()
        req.message = message
        req.severity = severity
        if event_type:
            req.type = event_type
        if time:
            req.time.MergeFrom(to_server_time(time))
        if source:
            req.source = source
        if sequence_number is not None:
            req.sequenceNumber = sequence_number
        if extra:
            for key in extra:
                req.extra[key] = extra[key]

        url = f"/archive/{instance}/events"
        self.ctx.post_proto(url, data=req.SerializeToString())

    def load_parameter_values(
        self,
        instance: str,
        data: Iterator[Mapping[str, "ValueUpdate"]],
        stream: str = "pp_dump",
        chunk_size: int = 32 * 1024,
    ) -> LoadParameterValuesResult:
        """
        Load an indefinite amount of parameter values into Yamcs.

        .. versionadded:: 1.9.1
           Compatible with Yamcs 5.8.8 onwards

        :param instance:
            Yamcs instance name
        :param data:
            Generator that yields batches of parameter values,
            keyed by parameter name.
        :param stream:
            Stream where to send the parameters to.
        :param chunk_size:
            HTTP chunk size. Multiple updates are grouped in chunks
            of about this size.
        """

        def to_proto(generator: Iterator[Mapping[str, "ValueUpdate"]]):
            for values in generator:
                proto = pvalue_service_pb2.LoadParameterValuesRequest()
                for key in values:
                    value_update = values[key]
                    item = proto.values.add()
                    item.parameter = key
                    item.value.MergeFrom(_build_value_proto(value_update.value))
                    if value_update.generation_time:
                        item.generationTime.MergeFrom(
                            to_server_time(value_update.generation_time)
                        )
                    if value_update.expires_in is not None:
                        item.expiresIn = int(value_update.expires_in * 1000)

                yield proto

        generator = delimit_protobuf(to_proto(data), chunk_size=chunk_size)

        url = f"/parameter-values/{instance}/streams/{stream}:load"
        response = self.ctx.post_proto(url, data=generator)
        message = pvalue_service_pb2.LoadParameterValuesResponse()
        message.ParseFromString(response.content)
        return LoadParameterValuesResult(message)

    def create_link_subscription(
        self,
        instance: str,
        on_data: Optional[Callable[[List[Link]], None]] = None,
        timeout: float = 60,
    ) -> LinkSubscription:
        """
        Create a new subscription for receiving data link updates of an instance.

        This method returns a future, then returns immediately. Stop the
        subscription by canceling the future.

        :param instance:
            A Yamcs instance name.
        :param on_data:
            Function that gets called with :class:`.Link` updates. Each update
            contains state of all links.
        :param timeout:
            The amount of seconds to wait for the request to complete.
        :return:
            Future that can be used to manage the background websocket
            subscription.
        """
        options = links_pb2.SubscribeLinksRequest()
        options.instance = instance
        manager = WebSocketSubscriptionManager(self.ctx, topic="links", options=options)

        # Represent subscription as a future
        subscription = LinkSubscription(manager)

        wrapped_callback = functools.partial(
            _wrap_callback_parse_link_event, subscription, on_data
        )

        manager.open(wrapped_callback)

        # Wait until a reply or exception is received
        subscription.reply(timeout=timeout)

        return subscription

    def create_time_subscription(
        self,
        instance: str,
        on_data: Optional[Callable[[datetime.datetime], None]] = None,
        timeout: float = 60,
    ) -> TimeSubscription:
        """
        Create a new subscription for receiving time updates of an instance.
        Time updates are emitted at 1Hz.

        This method returns a future, then returns immediately. Stop the
        subscription by canceling the future.

        :param instance:
            A Yamcs instance name
        :param on_data:
            Function that gets called with :class:`~datetime.datetime` updates.
        :param timeout:
            The amount of seconds to wait for the request to complete.

        :return:
            Future that can be used to manage the background websocket
            subscription.
        """
        options = time_service_pb2.SubscribeTimeRequest()
        options.instance = instance
        manager = WebSocketSubscriptionManager(self.ctx, topic="time", options=options)

        # Represent subscription as a future
        subscription = TimeSubscription(manager)

        wrapped_callback = functools.partial(
            _wrap_callback_parse_time_info, subscription, on_data
        )

        manager.open(wrapped_callback)

        # Wait until a reply or exception is received
        subscription.reply(timeout=timeout)

        return subscription

    def create_event_subscription(
        self,
        instance: str,
        on_data: Callable[[Event], None],
        filter: Optional[str] = None,
        timeout: float = 60,
    ) -> WebSocketSubscriptionFuture:
        """
        Create a new subscription for receiving events of an instance.

        This method returns a future, then returns immediately. Stop the
        subscription by canceling the future.

        :param instance:
            A Yamcs instance name
        :param on_data:
            Function that gets called on each :class:`.Event`.
        :param filter:
            Filter query applied to returned events. Allows for both text
            and field search.

            .. versionadded:: 1.10.1
               Compatible with Yamcs v5.10.2 onwards
        :param timeout:
            The amount of seconds to wait for the request to complete.
        :return:
            Future that can be used to manage the background websocket
            subscription.
        """
        options = events_service_pb2.SubscribeEventsRequest()
        options.instance = instance
        if filter is not None:
            options.filter = filter

        manager = WebSocketSubscriptionManager(
            self.ctx, topic="events", options=options
        )

        # Represent subscription as a future
        subscription = WebSocketSubscriptionFuture(manager)

        wrapped_callback = functools.partial(_wrap_callback_parse_event, on_data)

        manager.open(wrapped_callback)

        # Wait until a reply or exception is received
        subscription.reply(timeout=timeout)

        return subscription

    def close(self):
        """Close this client session"""
        self.ctx.close()



import warnings

warnings.warn(
    "The module 'yamcs.model' is deprecated. "
    "Import classes from 'yamcs.client' instead.",
    DeprecationWarning,
    stacklevel=2,
)

from yamcs.client.model import (  # noqa
    AuthInfo,
    Event,
    Instance,
    InstanceTemplate,
    Link,
    LinkAction,
    LoadParameterValuesResult,
    ObjectPrivilege,
    Processor,
    RdbTablespace,
    ServerInfo,
    Service,
    UserInfo,
)



import warnings

warnings.warn(
    "The module 'yamcs.core.exceptions' is deprecated. "
    "Import classes from 'yamcs.client' instead.",
    DeprecationWarning,
    stacklevel=2,
)

from yamcs.client.core.exceptions import (  # noqa
    ConnectionFailure,
    NotFound,
    TimeoutError,
    Unauthorized,
    YamcsError,
)



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: yamcs/api/annotations.proto

from google.protobuf.internal import enum_type_wrapper
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from google.protobuf import descriptor_pb2 as google_dot_protobuf_dot_descriptor__pb2


DESCRIPTOR = _descriptor.FileDescriptor(
  name='yamcs/api/annotations.proto',
  package='yamcs.api',
  syntax='proto2',
  serialized_options=b'\n\rorg.yamcs.apiB\020AnnotationsProtoP\001',
  serialized_pb=b'\n\x1byamcs/api/annotations.proto\x12\tyamcs.api\x1a google/protobuf/descriptor.proto\"\x9b\x02\n\tHttpRoute\x12\r\n\x03get\x18\x01 \x01(\tH\x00\x12\r\n\x03put\x18\x02 \x01(\tH\x00\x12\x0e\n\x04post\x18\x03 \x01(\tH\x00\x12\x10\n\x06\x64\x65lete\x18\x04 \x01(\tH\x00\x12\x0f\n\x05patch\x18\x05 \x01(\tH\x00\x12\x12\n\ndeprecated\x18\x06 \x01(\x08\x12\x0c\n\x04\x62ody\x18\x07 \x01(\t\x12\x15\n\rmax_body_size\x18\x08 \x01(\x05\x12\x11\n\toffloaded\x18\t \x01(\x08\x12\x17\n\x0f\x66ield_mask_root\x18\n \x01(\t\x12\x31\n\x13\x61\x64\x64itional_bindings\x18\x0b \x03(\x0b\x32\x14.yamcs.api.HttpRoute\x12\x0b\n\x03log\x18\x0c \x01(\t\x12\r\n\x05label\x18\r \x01(\tB\t\n\x07pattern\"z\n\x0eWebSocketTopic\x12\r\n\x05topic\x18\x01 \x01(\t\x12\x12\n\ndeprecated\x18\x02 \x01(\x08\x12\x36\n\x13\x61\x64\x64itional_bindings\x18\x03 \x03(\x0b\x32\x19.yamcs.api.WebSocketTopic\x12\r\n\x05label\x18\x04 \x01(\t*;\n\rFieldBehavior\x12\x1e\n\x1a\x46IELD_BEHAVIOR_UNSPECIFIED\x10\x00\x12\n\n\x06SECRET\x10\x01:/\n\x05label\x12\x1f.google.protobuf.ServiceOptions\x18\x80\x10 \x01(\t:2\n\x08markdown\x12\x1f.google.protobuf.ServiceOptions\x18\x8a\x10 \x01(\x08:D\n\x05route\x12\x1e.google.protobuf.MethodOptions\x18\xa1\x32 \x01(\x0b\x32\x14.yamcs.api.HttpRoute:M\n\twebsocket\x12\x1e.google.protobuf.MethodOptions\x18\xab\x32 \x01(\x0b\x32\x19.yamcs.api.WebSocketTopic:P\n\x0e\x66ield_behavior\x12\x1d.google.protobuf.FieldOptions\x18\x85\x33 \x03(\x0e\x32\x18.yamcs.api.FieldBehaviorB#\n\rorg.yamcs.apiB\x10\x41nnotationsProtoP\x01'
  ,
  dependencies=[google_dot_protobuf_dot_descriptor__pb2.DESCRIPTOR,])

_FIELDBEHAVIOR = _descriptor.EnumDescriptor(
  name='FieldBehavior',
  full_name='yamcs.api.FieldBehavior',
  filename=None,
  file=DESCRIPTOR,
  values=[
    _descriptor.EnumValueDescriptor(
      name='FIELD_BEHAVIOR_UNSPECIFIED', index=0, number=0,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='SECRET', index=1, number=1,
      serialized_options=None,
      type=None),
  ],
  containing_type=None,
  serialized_options=None,
  serialized_start=486,
  serialized_end=545,
)
_sym_db.RegisterEnumDescriptor(_FIELDBEHAVIOR)

FieldBehavior = enum_type_wrapper.EnumTypeWrapper(_FIELDBEHAVIOR)
FIELD_BEHAVIOR_UNSPECIFIED = 0
SECRET = 1

LABEL_FIELD_NUMBER = 2048
label = _descriptor.FieldDescriptor(
  name='label', full_name='yamcs.api.label', index=0,
  number=2048, type=9, cpp_type=9, label=1,
  has_default_value=False, default_value=b"".decode('utf-8'),
  message_type=None, enum_type=None, containing_type=None,
  is_extension=True, extension_scope=None,
  serialized_options=None, file=DESCRIPTOR)
MARKDOWN_FIELD_NUMBER = 2058
markdown = _descriptor.FieldDescriptor(
  name='markdown', full_name='yamcs.api.markdown', index=1,
  number=2058, type=8, cpp_type=7, label=1,
  has_default_value=False, default_value=False,
  message_type=None, enum_type=None, containing_type=None,
  is_extension=True, extension_scope=None,
  serialized_options=None, file=DESCRIPTOR)
ROUTE_FIELD_NUMBER = 6433
route = _descriptor.FieldDescriptor(
  name='route', full_name='yamcs.api.route', index=2,
  number=6433, type=11, cpp_type=10, label=1,
  has_default_value=False, default_value=None,
  message_type=None, enum_type=None, containing_type=None,
  is_extension=True, extension_scope=None,
  serialized_options=None, file=DESCRIPTOR)
WEBSOCKET_FIELD_NUMBER = 6443
websocket = _descriptor.FieldDescriptor(
  name='websocket', full_name='yamcs.api.websocket', index=3,
  number=6443, type=11, cpp_type=10, label=1,
  has_default_value=False, default_value=None,
  message_type=None, enum_type=None, containing_type=None,
  is_extension=True, extension_scope=None,
  serialized_options=None, file=DESCRIPTOR)
FIELD_BEHAVIOR_FIELD_NUMBER = 6533
field_behavior = _descriptor.FieldDescriptor(
  name='field_behavior', full_name='yamcs.api.field_behavior', index=4,
  number=6533, type=14, cpp_type=8, label=3,
  has_default_value=False, default_value=[],
  message_type=None, enum_type=None, containing_type=None,
  is_extension=True, extension_scope=None,
  serialized_options=None, file=DESCRIPTOR)


_HTTPROUTE = _descriptor.Descriptor(
  name='HttpRoute',
  full_name='yamcs.api.HttpRoute',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='get', full_name='yamcs.api.HttpRoute.get', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='put', full_name='yamcs.api.HttpRoute.put', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='post', full_name='yamcs.api.HttpRoute.post', index=2,
      number=3, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='delete', full_name='yamcs.api.HttpRoute.delete', index=3,
      number=4, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='patch', full_name='yamcs.api.HttpRoute.patch', index=4,
      number=5, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='deprecated', full_name='yamcs.api.HttpRoute.deprecated', index=5,
      number=6, type=8, cpp_type=7, label=1,
      has_default_value=False, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='body', full_name='yamcs.api.HttpRoute.body', index=6,
      number=7, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='max_body_size', full_name='yamcs.api.HttpRoute.max_body_size', index=7,
      number=8, type=5, cpp_type=1, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='offloaded', full_name='yamcs.api.HttpRoute.offloaded', index=8,
      number=9, type=8, cpp_type=7, label=1,
      has_default_value=False, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='field_mask_root', full_name='yamcs.api.HttpRoute.field_mask_root', index=9,
      number=10, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='additional_bindings', full_name='yamcs.api.HttpRoute.additional_bindings', index=10,
      number=11, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='log', full_name='yamcs.api.HttpRoute.log', index=11,
      number=12, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='label', full_name='yamcs.api.HttpRoute.label', index=12,
      number=13, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
    _descriptor.OneofDescriptor(
      name='pattern', full_name='yamcs.api.HttpRoute.pattern',
      index=0, containing_type=None, fields=[]),
  ],
  serialized_start=77,
  serialized_end=360,
)


_WEBSOCKETTOPIC = _descriptor.Descriptor(
  name='WebSocketTopic',
  full_name='yamcs.api.WebSocketTopic',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='topic', full_name='yamcs.api.WebSocketTopic.topic', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='deprecated', full_name='yamcs.api.WebSocketTopic.deprecated', index=1,
      number=2, type=8, cpp_type=7, label=1,
      has_default_value=False, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='additional_bindings', full_name='yamcs.api.WebSocketTopic.additional_bindings', index=2,
      number=3, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='label', full_name='yamcs.api.WebSocketTopic.label', index=3,
      number=4, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=362,
  serialized_end=484,
)

_HTTPROUTE.fields_by_name['additional_bindings'].message_type = _HTTPROUTE
_HTTPROUTE.oneofs_by_name['pattern'].fields.append(
  _HTTPROUTE.fields_by_name['get'])
_HTTPROUTE.fields_by_name['get'].containing_oneof = _HTTPROUTE.oneofs_by_name['pattern']
_HTTPROUTE.oneofs_by_name['pattern'].fields.append(
  _HTTPROUTE.fields_by_name['put'])
_HTTPROUTE.fields_by_name['put'].containing_oneof = _HTTPROUTE.oneofs_by_name['pattern']
_HTTPROUTE.oneofs_by_name['pattern'].fields.append(
  _HTTPROUTE.fields_by_name['post'])
_HTTPROUTE.fields_by_name['post'].containing_oneof = _HTTPROUTE.oneofs_by_name['pattern']
_HTTPROUTE.oneofs_by_name['pattern'].fields.append(
  _HTTPROUTE.fields_by_name['delete'])
_HTTPROUTE.fields_by_name['delete'].containing_oneof = _HTTPROUTE.oneofs_by_name['pattern']
_HTTPROUTE.oneofs_by_name['pattern'].fields.append(
  _HTTPROUTE.fields_by_name['patch'])
_HTTPROUTE.fields_by_name['patch'].containing_oneof = _HTTPROUTE.oneofs_by_name['pattern']
_WEBSOCKETTOPIC.fields_by_name['additional_bindings'].message_type = _WEBSOCKETTOPIC
DESCRIPTOR.message_types_by_name['HttpRoute'] = _HTTPROUTE
DESCRIPTOR.message_types_by_name['WebSocketTopic'] = _WEBSOCKETTOPIC
DESCRIPTOR.enum_types_by_name['FieldBehavior'] = _FIELDBEHAVIOR
DESCRIPTOR.extensions_by_name['label'] = label
DESCRIPTOR.extensions_by_name['markdown'] = markdown
DESCRIPTOR.extensions_by_name['route'] = route
DESCRIPTOR.extensions_by_name['websocket'] = websocket
DESCRIPTOR.extensions_by_name['field_behavior'] = field_behavior
_sym_db.RegisterFileDescriptor(DESCRIPTOR)

HttpRoute = _reflection.GeneratedProtocolMessageType('HttpRoute', (_message.Message,), {
  'DESCRIPTOR' : _HTTPROUTE,
  '__module__' : 'yamcs.api.annotations_pb2'
  # @@protoc_insertion_point(class_scope:yamcs.api.HttpRoute)
  })
_sym_db.RegisterMessage(HttpRoute)

WebSocketTopic = _reflection.GeneratedProtocolMessageType('WebSocketTopic', (_message.Message,), {
  'DESCRIPTOR' : _WEBSOCKETTOPIC,
  '__module__' : 'yamcs.api.annotations_pb2'
  # @@protoc_insertion_point(class_scope:yamcs.api.WebSocketTopic)
  })
_sym_db.RegisterMessage(WebSocketTopic)

google_dot_protobuf_dot_descriptor__pb2.ServiceOptions.RegisterExtension(label)
google_dot_protobuf_dot_descriptor__pb2.ServiceOptions.RegisterExtension(markdown)
route.message_type = _HTTPROUTE
google_dot_protobuf_dot_descriptor__pb2.MethodOptions.RegisterExtension(route)
websocket.message_type = _WEBSOCKETTOPIC
google_dot_protobuf_dot_descriptor__pb2.MethodOptions.RegisterExtension(websocket)
field_behavior.enum_type = _FIELDBEHAVIOR
google_dot_protobuf_dot_descriptor__pb2.FieldOptions.RegisterExtension(field_behavior)

DESCRIPTOR._options = None
# @@protoc_insertion_point(module_scope)



import abc
import datetime
from dataclasses import dataclass
from typing import Any, Dict, List, Optional, Union

from yamcs.client.activities import Activity, ManualActivity
from yamcs.client.core.helpers import ProtoList, parse_server_time, to_server_time
from yamcs.protobuf.timeline import timeline_pb2

__all__ = [
    "Band",
    "CommandBand",
    "Item",
    "ItemBand",
    "ParameterPlot",
    "ParameterStateBand",
    "RangeMapping",
    "Spacer",
    "TimeRuler",
    "Trace",
    "ValueMapping",
    "View",
]


class Item:
    def __init__(self, proto=None):
        merged = timeline_pb2.TimelineItem()
        merged.type = timeline_pb2.TimelineItemType.EVENT
        if proto:
            merged.MergeFrom(proto)
        self._proto = merged

    @property
    def id(self) -> str:
        """Item identifier."""
        return self._proto.id

    @property
    def item_type(self) -> str:
        """Type of item."""
        return timeline_pb2.TimelineItemType.Name(self._proto.type)

    @property
    def name(self) -> str:
        """Name of this item."""
        return self._proto.name

    @name.setter
    def name(self, value: str):
        self._proto.name = value

    @property
    def tags(self) -> List[str]:
        """
        Item tags. Used by bands to filter what is visible.
        """
        return self._proto.tags

    @tags.setter
    def tags(self, value: List[str]):
        self._proto.tags[:] = value

    @property
    def start(self) -> datetime.datetime:
        """
        Item start time.
        """
        return parse_server_time(self._proto.start)

    @start.setter
    def start(self, value: datetime.datetime):
        self._proto.start.MergeFrom(to_server_time(value))

    @property
    def duration(self) -> datetime.timedelta:
        """
        Item duration.
        """
        return self._proto.duration.ToTimedelta()

    @duration.setter
    def duration(self, value: datetime.timedelta):
        self._proto.duration.FromTimedelta(value)

    @property
    def activity(self) -> Optional[Activity]:
        """Activity definition."""
        if self._proto.HasField("activityDefinition"):
            return Activity._as_subclass(self._proto.activityDefinition)
        elif self._proto.type == timeline_pb2.TimelineItemType.ACTIVITY:
            return ManualActivity()
        return None

    @activity.setter
    def activity(self, value: Union[Activity, None]):
        if value is None:
            self._proto.type = timeline_pb2.TimelineItemType.EVENT
            self._proto.ClearField("activityDefinition")
        else:
            self._proto.type = timeline_pb2.TimelineItemType.ACTIVITY
            if isinstance(value, ManualActivity):
                self._proto.ClearField("activityDefinition")
            else:
                self._proto.activityDefinition.MergeFrom(value._to_proto())

    @property
    def background_color(self) -> Optional[str]:
        """CSS color string."""
        return self._proto.properties["backgroundColor"] or None

    @background_color.setter
    def background_color(self, value: Union[str, None]):
        if value is None:
            del self._proto.properties["backgroundColor"]
        else:
            self._proto.properties["backgroundColor"] = value

    @property
    def border_color(self) -> Optional[str]:
        """CSS color string."""
        return self._proto.properties["borderColor"] or None

    @border_color.setter
    def border_color(self, value: Union[str, None]):
        if value is None:
            del self._proto.properties["borderColor"]
        else:
            self._proto.properties["borderColor"] = value

    @property
    def border_width(self) -> Optional[int]:
        if "borderWidth" in self._proto.properties:
            return int(self._proto.properties["borderWidth"])
        return None

    @border_width.setter
    def border_width(self, value: Union[int, None]):
        if value is None:
            del self._proto.properties["borderWidth"]
        else:
            self._proto.properties["borderWidth"] = str(value)

    @property
    def corner_radius(self) -> Optional[int]:
        if "cornerRadius" in self._proto.properties:
            return int(self._proto.properties["cornerRadius"])
        return None

    @corner_radius.setter
    def corner_radius(self, value: Union[int, None]):
        if value is None:
            del self._proto.properties["cornerRadius"]
        else:
            self._proto.properties["cornerRadius"] = str(value)

    @property
    def margin_left(self) -> Optional[int]:
        if "marginLeft" in self._proto.properties:
            return int(self._proto.properties["marginLeft"])
        return None

    @margin_left.setter
    def margin_left(self, value: Union[int, None]):
        if value is None:
            del self._proto.properties["marginLeft"]
        else:
            self._proto.properties["marginLeft"] = str(value)

    @property
    def text_color(self) -> Optional[str]:
        """CSS color string."""
        return self._proto.properties["textColor"] or None

    @text_color.setter
    def text_color(self, value: Union[str, None]):
        if value is None:
            del self._proto.properties["textColor"]
        else:
            self._proto.properties["textColor"] = value

    @property
    def text_size(self) -> Optional[int]:
        if "textSize" in self._proto.properties:
            return int(self._proto.properties["textSize"])
        return None

    @text_size.setter
    def text_size(self, value: Union[int, None]):
        if value is None:
            del self._proto.properties["textSize"]
        else:
            self._proto.properties["textSize"] = str(value)

    def __str__(self):
        return self.name


class Band(abc.ABC):
    """
    Superclass for bands. Implementations:

    * :class:`.TimeRuler`
    * :class:`.ItemBand`
    * :class:`.ParameterPlot`
    * :class:`.ParameterStateBand`
    * :class:`.Spacer`
    * :class:`.CommandBand`
    """

    def __init__(self, proto):
        self._proto = proto

    @property
    def id(self) -> str:
        """Band identifier."""
        return self._proto.id

    @property
    def band_type(self) -> str:
        """Type of band."""
        return timeline_pb2.TimelineBandType.Name(self._proto.type)

    @property
    def name(self) -> str:
        """Name of this band."""
        return self._proto.name

    @name.setter
    def name(self, value: str):
        self._proto.name = value

    @property
    def description(self) -> str:
        """Description of this band."""
        return self._proto.description

    @description.setter
    def description(self, value: str):
        self._proto.description = value

    def _set_integer_property(self, key: str, value: int):
        if not isinstance(value, int):
            raise ValueError("Provided value is not integer")
        self._proto.properties[key] = str(value)

    def _get_integer_property(self, key: str):
        return int(self._proto.properties[key])

    def _set_optional_float_property(self, key: str, value: Optional[float]):
        if value is None:
            del self._proto.properties[key]
        else:
            if not isinstance(value, (int, float)):
                raise ValueError("Provided value is not float")
            self._proto.properties[key] = str(value)

    def _get_optional_float_property(self, key: Optional[str]) -> Optional[float]:
        return (
            float(self._proto.properties[key])
            if key in self._proto.properties
            else None
        )

    def _set_float_property(self, key: str, value: float):
        if not isinstance(value, (int, float)):
            raise ValueError("Provided value is not float")
        self._proto.properties[key] = str(value)

    def _get_float_property(self, key: str):
        return float(self._proto.properties[key])

    def _set_boolean_property(self, key: str, value: bool):
        if not isinstance(value, bool):
            raise ValueError("Provided value is not boolean")
        self._proto.properties[key] = "true" if value else "false"

    def _get_boolean_property(self, key: str):
        return self._proto.properties[key] == "true"

    def _as_properties(self) -> Dict[str, Any]:
        properties: Dict[str, Any] = {}
        for k, v in self._proto.properties.items():
            properties[k] = v
        return properties

    @staticmethod
    def _as_subclass(proto):
        if proto.type == timeline_pb2.TimelineBandType.TIME_RULER:
            return TimeRuler(proto)
        elif proto.type == timeline_pb2.TimelineBandType.ITEM_BAND:
            return ItemBand(proto)
        elif proto.type == timeline_pb2.TimelineBandType.SPACER:
            return Spacer(proto)
        elif proto.type == timeline_pb2.TimelineBandType.COMMAND_BAND:
            return CommandBand(proto)
        elif proto.type == timeline_pb2.TimelineBandType.PARAMETER_PLOT:
            return ParameterPlot(proto)
        elif proto.type == timeline_pb2.TimelineBandType.PARAMETER_STATES:
            return ParameterStateBand(proto)
        else:
            raise ValueError("Unexpected band type")

    def __str__(self):
        return self.name


class TimeRuler(Band):
    """
    Displays absolute time, formatted in a timezone of choice.
    """

    def __init__(self, proto=None):
        merged = timeline_pb2.TimelineBand()
        merged.type = timeline_pb2.TimelineBandType.TIME_RULER
        merged.properties["timezone"] = "UTC"
        if proto:
            merged.MergeFrom(proto)
        super(TimeRuler, self).__init__(merged)

    @property
    def timezone(self) -> str:
        """
        IANA timezone name.

        Corresponds with the third column of the following table:
        https://data.iana.org/time-zones/data/zone1970.tab

        In addition, the name `UTC` is supported.
        """
        return self._proto.properties["timezone"]

    @timezone.setter
    def timezone(self, value: str):
        self._proto.properties["timezone"] = value


class Spacer(Band):
    """
    Insert empty vertical space.
    """

    def __init__(self, proto=None):
        merged = timeline_pb2.TimelineBand()
        merged.type = timeline_pb2.TimelineBandType.SPACER
        merged.properties["height"] = "34"
        if proto:
            merged.MergeFrom(proto)
        super(Spacer, self).__init__(merged)

    @property
    def height(self) -> int:
        """Spacer height"""
        return self._get_integer_property("height")

    @height.setter
    def height(self, value: int):
        self._set_integer_property("height", value)


class CommandBand(Band):
    """
    Display issued commands.
    """

    def __init__(self, proto=None):
        merged = timeline_pb2.TimelineBand()
        merged.type = timeline_pb2.TimelineBandType.COMMAND_BAND
        if proto:
            merged.MergeFrom(proto)
        super(CommandBand, self).__init__(merged)


@dataclass
class Trace:
    """
    A trace on a :class:`.ParameterPlot`.
    """

    parameter: str
    line_color: str
    visible: bool = True
    line_width: int = 1
    fill: bool = False
    fill_color: str = "#dddddd"
    min_max: bool = False
    min_max_opacity: float = 0.17


class ParameterPlot(Band):
    """
    Plot the values of a numeric parameter.

    .. versionadded:: 1.11.2
       Compatible with Yamcs 5.11.2 onwards
    """

    def __init__(self, proto=None):
        merged = timeline_pb2.TimelineBand()
        merged.type = timeline_pb2.TimelineBandType.PARAMETER_PLOT
        merged.properties["frozen"] = "false"
        merged.properties["height"] = "30"
        merged.properties["zeroLineWidth"] = "0"
        merged.properties["zeroLineColor"] = "#ff0000"
        merged.properties["minimumFractionDigits"] = "0"
        merged.properties["maximumFractionDigits"] = "2"

        self.traces: List[Trace] = []
        """
        Plot lines.
        """

        if proto:
            merged.MergeFrom(proto)

            idx = 1
            while True:
                if f"trace_{idx}_type" in merged.properties:
                    prefix = f"trace_{idx}_"
                    trace = Trace(
                        parameter=merged.properties.get(f"{prefix}parameter"),
                        line_color=merged.properties.get(f"{prefix}lineColor"),
                        visible=bool(merged.properties.get(f"{prefix}visible")),
                        line_width=int(merged.properties.get(f"{prefix}lineWidth")),
                        fill=bool(merged.properties.get(f"{prefix}fill")),
                        fill_color=merged.properties.get(f"{prefix}fillColor"),
                        min_max=bool(merged.properties.get(f"{prefix}minMax")),
                        min_max_opacity=float(
                            merged.properties.get(f"{prefix}minMaxOpacity")
                        ),
                    )
                    self.traces.append(trace)
                    idx += 1
                else:
                    break

        super(ParameterPlot, self).__init__(merged)

    @property
    def frozen(self) -> bool:
        """
        Fix this line to the top of the view. Frozen bands are always
        rendered above other bands.
        """
        return self._get_boolean_property("frozen")

    @frozen.setter
    def frozen(self, value: bool):
        self._set_boolean_property("frozen", value)

    @property
    def height(self) -> int:
        """Band height"""
        return self._get_integer_property("height")

    @height.setter
    def height(self, value: int):
        self._set_integer_property("height", value)

    @property
    def minimum(self) -> Optional[float]:
        """Minimum value to show on Y-axis. Set to ``None`` for fitting actual data"""
        return self._get_optional_float_property("minimum")

    @minimum.setter
    def minimum(self, value: Optional[float]):
        self._set_optional_float_property("minimum", value)

    @property
    def maximum(self) -> Optional[float]:
        """Maximum value to show on Y-axis. Set to ``None`` for fitting actual data"""
        return self._get_optional_float_property("maximum")

    @maximum.setter
    def maximum(self, value: Optional[float]):
        self._set_optional_float_property("maximum", value)

    @property
    def zero_line_width(self) -> int:
        """Thickness of the zero line. 0 is invisible"""
        return self._get_integer_property("zeroLineWidth")

    @zero_line_width.setter
    def zero_line_width(self, value: int):
        self._set_integer_property("zeroLineWidth", value)

    @property
    def zero_line_color(self) -> str:
        """Color of the zero line"""
        return self._proto.properties["zeroLineColor"]

    @zero_line_color.setter
    def zero_line_color(self, value: str):
        self._proto.properties["zeroLineColor"] = value

    @property
    def minimum_fraction_digits(self) -> int:
        """Minimum fraction digits"""
        return self._get_integer_property("minimumFractionDigits")

    @minimum_fraction_digits.setter
    def minimum_fraction_digits(self, value: int):
        self._set_integer_property("minimumFractionDigits", value)

    @property
    def maximum_fraction_digits(self) -> int:
        """Maximum fraction digits"""
        return self._get_integer_property("maximumFractionDigits")

    @maximum_fraction_digits.setter
    def maximum_fraction_digits(self, value: int):
        self._set_integer_property("maximumFractionDigits", value)

    def _as_properties(self) -> Dict[str, Any]:
        props = super(ParameterPlot, self)._as_properties()
        for index, trace in enumerate(self.traces):
            props[f"trace_{index + 1}_parameter"] = trace.parameter
            props[f"trace_{index + 1}_lineColor"] = trace.line_color
            props[f"trace_{index + 1}_visible"] = "true" if trace.visible else "false"
            props[f"trace_{index + 1}_lineWidth"] = str(trace.line_width)
            props[f"trace_{index + 1}_fill"] = "true" if trace.fill else "false"
            props[f"trace_{index + 1}_fillColor"] = trace.fill_color
            props[f"trace_{index + 1}_minMax"] = "true" if trace.min_max else "false"
            props[f"trace_{index + 1}_minMaxOpacity"] = str(trace.min_max_opacity)
        return props


@dataclass
class ValueMapping:
    """
    Maps a value to a label and/or color
    """

    value: Any
    """
    Engineering value to match
    """

    label: Optional[str] = None
    """
    If specified, map the provided value to this label
    """

    color: Optional[str] = None
    """
    If specified, show states of this value (or mapped label) in this color
    """


@dataclass
class RangeMapping:
    """
    Maps a value to a label and/or color.
    """

    start: float
    """
    Match engineering value greater or equal than the provided start value
    """

    end: float
    """
    Match engineering value lesser or equal than the provided end value
    """

    label: Optional[str] = None
    """
    If specified, map the provided value to this label
    """

    color: Optional[str] = None
    """
    If specified, show states of this value (or mapped label) in this color
    """


class ParameterStateBand(Band):
    """
    Show state transitions of a parameter

    .. versionadded:: 1.11.2
       Compatible with Yamcs 5.11.2 onwards
    """

    def __init__(self, proto=None):
        merged = timeline_pb2.TimelineBand()
        merged.type = timeline_pb2.TimelineBandType.PARAMETER_STATES
        merged.properties["frozen"] = "false"
        merged.properties["height"] = "30"
        merged.properties["parameter"] = ""

        self.mappings: List[Union[ValueMapping, RangeMapping]] = []
        """
        Map engineering values to a label and/or color. Mappings are applied
        in order.
        """

        if proto:
            merged.MergeFrom(proto)

            idx = 0
            while True:
                if f"value_mapping_{idx}_type" in merged.properties:
                    prefix = f"value_mapping_{idx}_"
                    type_ = merged.properties[f"{prefix}type"]
                    if type_ == "value":
                        mapping = ValueMapping(
                            value=merged.properties.get(f"{prefix}value"),
                            label=merged.properties.get(f"{prefix}label"),
                            color=merged.properties.get(f"{prefix}color"),
                        )
                        self.mappings.append(mapping)
                    elif type_ == "range":
                        mapping = RangeMapping(
                            start=float(merged.properties.get(f"{prefix}start")),
                            end=float(merged.properties.get(f"{prefix}end")),
                            label=merged.properties.get(f"{prefix}label"),
                            color=merged.properties.get(f"{prefix}color"),
                        )
                        self.mappings.append(mapping)
                    else:
                        raise ValueError(f"Unexpected mapping type '{type_}'")
                    idx += 1
                else:
                    break

        super(ParameterStateBand, self).__init__(merged)

    @property
    def frozen(self) -> bool:
        """
        Fix this line to the top of the view. Frozen bands are always
        rendered above other bands.
        """
        return self._get_boolean_property("frozen")

    @frozen.setter
    def frozen(self, value: bool):
        self._set_boolean_property("frozen", value)

    @property
    def height(self) -> int:
        """Band height"""
        return self._get_integer_property("height")

    @height.setter
    def height(self, value: int):
        self._set_integer_property("height", value)

    @property
    def parameter(self) -> str:
        """Qualified parameter name"""
        return self._proto.properties["parameter"]

    @parameter.setter
    def parameter(self, value: str):
        self._proto.properties["parameter"] = value

    def _as_properties(self) -> Dict[str, Any]:
        props = super(ParameterStateBand, self)._as_properties()
        for index, mapping in enumerate(self.mappings):
            if isinstance(mapping, ValueMapping):
                props[f"value_mapping_{index}_type"] = "value"
                props[f"value_mapping_{index}_value"] = str(mapping.value)
                if mapping.label is not None:
                    props[f"value_mapping_{index}_label"] = str(mapping.label)
                if mapping.color is not None:
                    props[f"value_mapping_{index}_color"] = str(mapping.color)
            elif isinstance(mapping, RangeMapping):
                props[f"value_mapping_{index}_type"] = "range"
                props[f"value_mapping_{index}_start"] = str(mapping.start)
                props[f"value_mapping_{index}_end"] = str(mapping.end)
                if mapping.label is not None:
                    props[f"value_mapping_{index}_label"] = str(mapping.label)
                if mapping.color is not None:
                    props[f"value_mapping_{index}_color"] = str(mapping.color)
        return props


class ItemBand(Band):
    """
    Show a selection of timeline items.
    """

    def __init__(self, proto=None):
        merged = timeline_pb2.TimelineBand()
        merged.type = timeline_pb2.TimelineBandType.ITEM_BAND
        merged.properties["frozen"] = "false"
        merged.properties["itemBackgroundColor"] = "#77b1e1"
        merged.properties["itemBorderColor"] = "#3d94c7"
        merged.properties["itemBorderWidth"] = "1"
        merged.properties["itemCornerRadius"] = "0"
        merged.properties["itemHeight"] = "20"
        merged.properties["itemMarginLeft"] = "5"
        merged.properties["itemTextColor"] = "#333333"
        merged.properties["itemTextOverflow"] = "show"
        merged.properties["itemTextSize"] = "10"
        merged.properties["marginTop"] = "7"
        merged.properties["marginBottom"] = "7"
        merged.properties["multiline"] = "true"
        merged.properties["spaceBetweenItems"] = "0"
        merged.properties["spaceBetweenLines"] = "2"
        if proto:
            merged.MergeFrom(proto)
        super(ItemBand, self).__init__(merged)

    @property
    def frozen(self) -> bool:
        """
        Fix this line to the top of the view. Frozen bands are always
        rendered above other bands.
        """
        return self._get_boolean_property("frozen")

    @frozen.setter
    def frozen(self, value: bool):
        self._set_boolean_property("frozen", value)

    @property
    def tags(self) -> List[str]:
        """
        Item tags that this band filters on.
        """
        return self._proto.tags

    @tags.setter
    def tags(self, value: List[str]):
        self._proto.tags[:] = value

    @property
    def item_background_color(self) -> str:
        """CSS color string."""
        return self._proto.properties["itemBackgroundColor"]

    @item_background_color.setter
    def item_background_color(self, value: str):
        self._proto.properties["itemBackgroundColor"] = value

    @property
    def item_border_color(self) -> str:
        """CSS color string."""
        return self._proto.properties["itemBorderColor"]

    @item_border_color.setter
    def item_border_color(self, value: str):
        self._proto.properties["itemBorderColor"] = value

    @property
    def item_border_width(self) -> int:
        return self._get_integer_property("itemBorderWidth")

    @item_border_width.setter
    def item_border_width(self, value: int):
        self._set_integer_property("itemBorderWidth", value)

    @property
    def item_corner_radius(self) -> int:
        return self._get_integer_property("itemCornerRadius")

    @item_corner_radius.setter
    def item_corner_radius(self, value: int):
        self._set_integer_property("itemCornerRadius", value)

    @property
    def item_height(self) -> int:
        return self._get_integer_property("itemHeight")

    @item_height.setter
    def item_height(self, value: int):
        self._set_integer_property("itemHeight", value)

    @property
    def item_margin_left(self) -> int:
        return self._get_integer_property("itemMarginLeft")

    @item_margin_left.setter
    def item_margin_left(self, value: int):
        self._set_integer_property("itemMarginLeft", value)

    @property
    def item_text_color(self) -> str:
        """CSS color string."""
        return self._proto.properties["itemTextColor"]

    @item_text_color.setter
    def item_text_color(self, value: str):
        self._proto.properties["itemTextColor"] = value

    @property
    def item_text_overflow(self) -> str:
        """One of ``show``, ``clip``, or ``hide``."""
        return self._proto.properties["itemTextOverflow"]

    @item_text_overflow.setter
    def item_text_overflow(self, value: str):
        self._proto.properties["itemTextOverflow"] = value

    @property
    def item_text_size(self) -> int:
        return self._get_integer_property("itemTextSize")

    @item_text_size.setter
    def item_text_size(self, value: int):
        self._set_integer_property("itemTextSize", value)

    @property
    def margin_bottom(self) -> int:
        return self._get_integer_property("marginBottom")

    @margin_bottom.setter
    def margin_bottom(self, value: int):
        self._set_integer_property("marginBottom", value)

    @property
    def margin_top(self) -> int:
        return self._get_integer_property("marginTop")

    @margin_top.setter
    def margin_top(self, value: int):
        self._set_integer_property("marginTop", value)

    @property
    def multiline(self) -> bool:
        """
        Draw items on multiple lines if otherwise there would be collisions.
        """
        return self._get_boolean_property("multiline")

    @multiline.setter
    def multiline(self, value: bool):
        self._set_boolean_property("multiline", value)

    @property
    def space_between_items(self) -> int:
        """
        In case of multilining, this indicates the minimum horizontal space
        between items. If an item does not meet this treshold, it gets
        rendered on a different line.
        """
        return self._get_integer_property("spaceBetweenItems")

    @space_between_items.setter
    def space_between_items(self, value: int):
        self._set_integer_property("spaceBetweenItems", value)

    @property
    def space_between_lines(self) -> int:
        """
        In case of multilining, this indicates the vertical space between
        lines.
        """
        return self._get_integer_property("spaceBetweenLines")

    @space_between_lines.setter
    def space_between_lines(self, value: int):
        self._set_integer_property("spaceBetweenLines", value)


class View:
    def __init__(self, proto=None):
        merged = timeline_pb2.TimelineView()
        if proto:
            merged.MergeFrom(proto)
        self._proto = merged

        self._bands = ProtoList(self._proto, "bands", lambda x: Band._as_subclass(x))

    @property
    def id(self) -> str:
        """View identifier."""
        return self._proto.id

    @property
    def name(self) -> str:
        """Name of this view."""
        return self._proto.name

    @name.setter
    def name(self, value: str):
        self._proto.name = value

    @property
    def description(self) -> str:
        """Description of this view."""
        return self._proto.description

    @description.setter
    def description(self, value: str):
        self._proto.name = value

    @property
    def bands(self) -> List[Band]:
        """
        Bands included in this view.
        """
        return self._bands

    @bands.setter
    def bands(self, value: List[Band]):
        self._bands.clear()
        self._bands.extend(value)

    def __str__(self):
        return self.name



import warnings

warnings.warn(
    "The module 'yamcs.timeline.model' is deprecated. "
    "Import classes from 'yamcs.client' instead.",
    DeprecationWarning,
    stacklevel=2,
)

from yamcs.client.timeline.model import (  # noqa
    Band,
    CommandBand,
    Item,
    ItemBand,
    Spacer,
    TimeRuler,
    View,
)



import warnings

warnings.warn(
    "The module 'yamcs.core.auth' is deprecated. "
    "Import classes from 'yamcs.client' instead.",
    DeprecationWarning,
    stacklevel=2,
)

from yamcs.client.core.auth import (  # noqa
    APIKeyCredentials,
    BasicAuthCredentials,
    Credentials,
)



import datetime
from typing import List, Optional

from yamcs.client.core.helpers import parse_server_time

__all__ = [
    "TCOStatus",
    "TCOCoefficients",
    "TCOSample",
    "TofInterval",
]


class TCOStatus:
    """
    TCO Status
    """

    def __init__(self, proto):
        self._proto = proto

    @property
    def coefficients(self) -> Optional["TCOCoefficients"]:
        """
        Current coefficients. Or ``None`` if the synchronization is not
        yet established.
        """
        if self._proto.HasField("coefficients"):
            return TCOCoefficients(self._proto.coefficients)
        return None

    @property
    def coefficients_time(self) -> Optional[datetime.datetime]:
        """
        Time when the coefficients have been computed
        """
        if self._proto.HasField("coefficientsTime"):
            return parse_server_time(self._proto.coefficientsTime)
        return None

    @property
    def deviation(self) -> Optional[float]:
        """
        Last computed deviation
        """
        if self._proto.HasField("deviation"):
            return self._proto.deviation
        return None

    @property
    def samples(self) -> List["TCOSample"]:
        """
        The last accumulated samples
        """
        return [TCOSample(p) for p in self._proto.samples]


class TCOCoefficients:
    """
    TCO Coefficients
    """

    def __init__(self, proto):
        self._proto = proto

    @property
    def utc(self) -> Optional[datetime.datetime]:
        if self._proto.HasField("utc"):
            return parse_server_time(self._proto.utc)
        return None

    @property
    def obt(self) -> Optional[int]:
        if self._proto.HasField("obt"):
            return self._proto.obt
        return None

    @property
    def gradient(self) -> Optional[float]:
        if self._proto.HasField("gradient"):
            return self._proto.gradient
        return None

    @property
    def offset(self) -> Optional[float]:
        if self._proto.HasField("offset"):
            return self._proto.offset
        return None


class TCOSample:
    def __init__(self, proto):
        self._proto = proto

    @property
    def utc(self) -> Optional[datetime.datetime]:
        if self._proto.HasField("utc"):
            return parse_server_time(self._proto.utc)
        return None

    @property
    def obt(self) -> Optional[int]:
        if self._proto.HasField("obt"):
            return self._proto.obt
        return None

    def __str__(self):
        return f"({self.utc}, {self.obt})"


class TofInterval:
    """
    ToF interval for the ERT range ``[start, stop]``, specifying
    a polynomial function of the form: `tof = a + bx + cx^2 + ...` where `x`
    is ERT minus the provided start date.
    """

    def __init__(
        self, start: datetime.datetime, stop: datetime.datetime, polynomial: List[float]
    ):
        """
        :param start:
            ERT start
        :param stop:
            ERT stop
        :param polynomial:
            Coefficients in the order ``[a, b, c, ...]``
        """
        self.start = start
        self.stop = stop
        self.polynomial = polynomial
